@@ -1,7 +1,40 @@
 /* 
-   SDL port of Atari800
-   Jacek Poplawski <jacekp@linux.com.pl>
+ * atari_sdl.c - SDL library specific port code
+ *
+ * Copyright (c) 2001-2002 Jacek Poplawski
+ * Copyright (C) 2001-2003 Atari800 development team (see DOC/CREDITS)
+ *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
 
+/*
+   Changelog:
+
+   09-02-2002 - "-mzpokey", "-oldpokey" and "-stereo" removed
+
+   14-12-2002 - added support for new audio stuff:
+   		"-mzpokey" - enable new (better) pokey code
+		"-oldpokey" - disable new pokey code (default)
+		"-stereo" - enable stereo mode
+		"-audio16" - enable 16bit audio mode (recommended)
+   23-04-2002 - added command line options:
+		"--nojoystick", "--width x", "--height y", "--bpp b",
+		"--fullscreen", "--windowed"
    06-12-2001 - replaced SHORTER_MODE with WIDTH_MODE, now there are 3 states:
 		DEFAULT_WIDTH_MODE (ATARI_WIDTH-2*24), SHORTER_WIDTH_MODE
 		(ATARI_WIDTH-2*24-2*8), and FULL_WIDTH_MODE (ATARI_WIDTH)
@@ -83,12 +116,24 @@
      will play without scaling (speed!)  
 
    Thanks to David Olofson for scaling tips!  
-
 */
 
 #include <SDL.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+
+#ifdef linux
+#define LPTJOY	1
+#endif
+
+#ifdef LPTJOY
+#include <fcntl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/ioctl.h>
+#include <linux/lp.h>
+#endif /* LPTJOY */
 
 // Atari800 includes
 #include "config.h"
@@ -103,19 +148,24 @@
 #include "diskled.h"
 #include "devices.h"
 #include "cpu.h"
-#include "memory-d.h"
+#include "memory.h"
 #include "pia.h"
 #include "log.h"
 
 // you can set that variables in code, or change it when emulator is running
 // I am not sure what to do with sound_enabled (can't turn it on inside
 // emulator, probably we need two variables or command line argument)
-static int sound_enabled = 1;
+#ifdef SOUND
+static int sound_enabled = TRUE;
+static int sound_flags = 0;
+static int sound_bits = 8;
+#endif
 static int SDL_ATARI_BPP = 0;	// 0 - autodetect
 static int FULLSCREEN = 1;
 static int BW = 0;
 static int SWAP_JOYSTICKS = 0;
 static int WIDTH_MODE = 1;
+static int ROTATE90 =0;
 #define SHORT_WIDTH_MODE 0
 #define DEFAULT_WIDTH_MODE 1
 #define FULL_WIDTH_MODE 2
@@ -151,35 +201,43 @@ int SDL_JOY_1_RIGHTDOWN = SDLK_c;
 
 // real joysticks
 
-SDL_Joystick *joystick0, *joystick1;
+int fd_joystick0 = -1;
+int fd_joystick1 = -1;
+
+SDL_Joystick *joystick0 = NULL;
+SDL_Joystick *joystick1 = NULL;
 int joystick0_nbuttons, joystick1_nbuttons;
 
 #define minjoy 10000			// real joystick tolerancy
 
-extern int refresh_rate;
-
 extern int alt_function;
 
+#ifdef SOUND
 // sound 
 #define FRAGSIZE        10		// 1<<FRAGSIZE is size of sound buffer
 static int SOUND_VOLUME = SDL_MIX_MAXVOLUME / 4;
-#define dsprate 44100
+static int dsprate = 44100;
+#endif
 
 // video
 SDL_Surface *MainScreen = NULL;
 SDL_Color colors[256];			// palette
 Uint16 Palette16[256];			// 16-bit palette
 Uint32 Palette32[256];			// 32-bit palette
-#define CurrentBPP -1
 
 // keyboard
 Uint8 *kbhits;
 static int last_key_break = 0;
 static int last_key_code = AKEY_NONE;
 
-int Sound_Update(void) { return 0; }	// fake function
+#ifdef SOUND
+int Sound_Update(void)
+{
+	return 0;
+}								// fake function
+#endif
 
-void SetPalette()
+static void SetPalette()
 {
 	SDL_SetPalette(MainScreen, SDL_LOGPAL | SDL_PHYSPAL, colors, 0, 256);
 }
@@ -274,6 +332,15 @@ void SetVideoMode(int w, int h, int bpp)
 void SetNewVideoMode(int w, int h, int bpp)
 {
 	float ww, hh;
+
+	if (ROTATE90)
+	{
+		SetVideoMode(w,h,bpp);
+	}
+	else
+	{
+	
+
 	if ((h < ATARI_HEIGHT) || (w < ATARI_WIDTH)) {
 		h = ATARI_HEIGHT;
 		w = ATARI_WIDTH;
@@ -309,9 +376,6 @@ void SetNewVideoMode(int w, int h, int bpp)
 	h = h / 8;
 	h = h * 8;
 
-	if (bpp == CurrentBPP)
-		bpp = MainScreen->format->BitsPerPixel;
-
 	SetVideoMode(w, h, bpp);
 
 	SDL_ATARI_BPP = MainScreen->format->BitsPerPixel;
@@ -324,6 +388,7 @@ void SetNewVideoMode(int w, int h, int bpp)
 			SetVideoMode(w, h, 8);
 		}
 	}
+	}
 
 	SetPalette();
 
@@ -336,7 +401,8 @@ void SetNewVideoMode(int w, int h, int bpp)
 void SwitchFullscreen()
 {
 	FULLSCREEN = 1 - FULLSCREEN;
-	SetNewVideoMode(MainScreen->w, MainScreen->h, CurrentBPP);
+	SetNewVideoMode(MainScreen->w, MainScreen->h,
+					MainScreen->format->BitsPerPixel);
 	Atari_DisplayScreen((UBYTE *) atari_screen);
 }
 
@@ -345,16 +411,8 @@ void SwitchWidth()
 	WIDTH_MODE++;
 	if (WIDTH_MODE > FULL_WIDTH_MODE)
 		WIDTH_MODE = SHORT_WIDTH_MODE;
-	SetNewVideoMode(MainScreen->w, MainScreen->h, CurrentBPP);
-	Atari_DisplayScreen((UBYTE *) atari_screen);
-}
-
-void SwitchBPP()
-{
-	SDL_ATARI_BPP *= 2;
-	if (SDL_ATARI_BPP > 32)
-		SDL_ATARI_BPP = 8;
-	SetNewVideoMode(MainScreen->w, MainScreen->h, SDL_ATARI_BPP);
+	SetNewVideoMode(MainScreen->w, MainScreen->h,
+					MainScreen->format->BitsPerPixel);
 	Atari_DisplayScreen((UBYTE *) atari_screen);
 }
 
@@ -372,21 +430,63 @@ void SwapJoysticks()
 	ModeInfo();
 }
 
+#ifdef SOUND
 void SDL_Sound_Update(void *userdata, Uint8 * stream, int len)
 {
-	Uint8 dsp_buffer[1 << FRAGSIZE];
+	Uint8 dsp_buffer[2 << FRAGSIZE]; // x2, because 16bit buffers
 	if (len > 1 << FRAGSIZE)
 		len = 1 << FRAGSIZE;
 	Pokey_process(dsp_buffer, len);
-	SDL_MixAudio(stream, dsp_buffer, len, SOUND_VOLUME);
+	if (sound_bits==8)
+		SDL_MixAudio(stream, dsp_buffer, len, SOUND_VOLUME);
+	else	
+		SDL_MixAudio(stream, dsp_buffer, 2*len, SOUND_VOLUME);
 }
 
 void SDL_Sound_Initialise(int *argc, char *argv[])
 {
+	int i, j;
 	SDL_AudioSpec desired, obtained;
+
+	for (i = j = 1; i < *argc; i++) {
+		if (strcmp(argv[i], "-sound") == 0)
+			sound_enabled = TRUE;
+		else if (strcmp(argv[i], "-nosound") == 0)
+			sound_enabled = FALSE;
+		else if (strcmp(argv[i], "-audio16") == 0)
+		{	
+			Aprint("audio 16bit enabled");
+			sound_flags|=SND_BIT16;
+			sound_bits=16;
+		}	
+		else if (strcmp(argv[i], "-dsprate") == 0)
+			sscanf(argv[++i], "%d", &dsprate);
+		else {
+			if (strcmp(argv[i], "-help") == 0) {
+				Aprint("\t-sound           Enable sound\n"
+				       "\t-nosound         Disable sound\n"
+				       "\t-dsprate <rate>  Set DSP rate in Hz\n"
+				      );
+			}
+			argv[j++] = argv[i];
+		}
+	}
+	*argc = j;
+
 	if (sound_enabled) {
 		desired.freq = dsprate;
-		desired.format = AUDIO_S8;
+		if (sound_bits==8)
+			desired.format = AUDIO_U8;
+		else
+			if (sound_bits==16)
+				desired.format = AUDIO_U16;
+		else
+		{
+			Aprint ("unknown sound_bits");
+			Atari800_Exit(FALSE);
+			Aflushlog();
+		};	
+			
 		desired.samples = 1 << FRAGSIZE;
 		desired.callback = SDL_Sound_Update;
 		desired.userdata = NULL;
@@ -394,24 +494,23 @@ void SDL_Sound_Initialise(int *argc, char *argv[])
 
 		if (SDL_OpenAudio(&desired, &obtained) < 0) {
 			Aprint("Problem with audio: %s", SDL_GetError());
-			Aprint("You can disable sound by setting sound_enabled=0");
-			Aflushlog();
-			exit(-1);
+			Aprint("Sound is disabled.");
+			sound_enabled = FALSE;
+			return;
 		}
 
-		// mono
-		Pokey_sound_init(FREQ_17_EXACT, dsprate, 1);
-		Aprint("sound initialized");
+		Pokey_sound_init(FREQ_17_EXACT, dsprate, 1, sound_flags);
 	}
 	else {
 		Aprint
-			("Audio is off, you can turn it on by setting sound_enabled=1");
+			("Audio is off, you can turn it by -sound");
 	}
 }
+#endif
 
 int Atari_Keyboard(void)
 {
-	static int lastkey = AKEY_NONE, key_pressed = 0;
+	static int lastkey = AKEY_NONE, key_pressed = 0, key_control = 0;
 
 	SDL_Event event;
 	if (SDL_PollEvent(&event)) {
@@ -425,7 +524,8 @@ int Atari_Keyboard(void)
 			key_pressed = 0;
 			break;
 		case SDL_VIDEORESIZE:
-			SetNewVideoMode(event.resize.w, event.resize.h, CurrentBPP);
+			SetNewVideoMode(event.resize.w, event.resize.h,
+							MainScreen->format->BitsPerPixel);
 			break;
 		case SDL_QUIT:
 			return AKEY_EXIT;
@@ -444,9 +544,6 @@ int Atari_Keyboard(void)
 	if (kbhits[SDLK_LALT]) {
 		if (key_pressed) {
 			switch (lastkey) {
-			case SDLK_e:
-				SwitchBPP();
-				break;
 			case SDLK_f:
 				SwitchFullscreen();
 				break;
@@ -502,6 +599,12 @@ int Atari_Keyboard(void)
 	else
 		key_shift = 0;
 
+    // CONTROL STATE
+	if ((kbhits[SDLK_LCTRL]) || (kbhits[SDLK_RCTRL]))
+		key_control = 1;
+	else
+		key_control = 0;
+
 	// OPTION / SELECT / START keys
 	key_consol = CONSOL_NONE;
 	if (kbhits[SDLK_F2])
@@ -585,8 +688,8 @@ int Atari_Keyboard(void)
 			return AKEY_DOLLAR;
 		case SDLK_5:
 			return AKEY_PERCENT;
-//      case SDLK_6:
-//          return AKEY_;    ^ <-- what is that?
+		case SDLK_6:
+			return AKEY_CARET;
 		case SDLK_7:
 			return AKEY_AMPERSAND;
 		case SDLK_8:
@@ -719,6 +822,10 @@ int Atari_Keyboard(void)
 	switch (lastkey) {
 	case SDLK_END:
 		return AKEY_HELP;
+	case SDLK_PAGEDOWN:
+		return AKEY_HELP;
+	case SDLK_PAGEUP:
+		return AKEY_CAPSLOCK;
 	case SDLK_HOME:
 		return AKEY_CLEAR;
 	case SDLK_PAUSE:
@@ -746,67 +853,239 @@ int Atari_Keyboard(void)
 	case SDLK_ESCAPE:
 		return AKEY_ESCAPE;
 	case SDLK_TAB:
-		return AKEY_TAB;
+		if (key_shift)
+			return AKEY_SETTAB;
+		else if (key_control)
+			return AKEY_CLRTAB;
+		else
+			return AKEY_TAB;
+	case SDLK_DELETE:
+		if (key_shift)
+			return AKEY_DELETE_LINE;
+		else
+			return AKEY_DELETE_CHAR;
+	case SDLK_INSERT:
+		if (key_shift)
+			return AKEY_INSERT_LINE;
+		else
+			return AKEY_INSERT_CHAR;
 	}
 	return AKEY_NONE;
 }
 
-void Init_Joysticks()
+void Init_SDL_Joysticks(int first, int second)
 {
-	joystick0 = SDL_JoystickOpen(0);
-	if (joystick0 == NULL)
-		Aprint("joystick 0 not found");
-	else {
-		Aprint("joystick 0 found!");
-		joystick0_nbuttons = SDL_JoystickNumButtons(joystick0);
-		SWAP_JOYSTICKS = 1;	// real joy is STICK(0) and numblock is STICK(1)
+	if (first) {
+		joystick0 = SDL_JoystickOpen(0);
+		if (joystick0 == NULL)
+			Aprint("joystick 0 not found");
+		else {
+			Aprint("joystick 0 found!");
+			joystick0_nbuttons = SDL_JoystickNumButtons(joystick0);
+			SWAP_JOYSTICKS = 1;		// real joy is STICK(0) and numblock is STICK(1)
+		}
 	}
-	joystick1 = SDL_JoystickOpen(1);
-	if (joystick1 == NULL)
-		Aprint("joystick 1 not found");
-	else {
-		Aprint("joystick 1 found!");
-		joystick1_nbuttons = SDL_JoystickNumButtons(joystick1);
+
+	if (second) {
+		joystick1 = SDL_JoystickOpen(1);
+		if (joystick1 == NULL)
+			Aprint("joystick 1 not found");
+		else {
+			Aprint("joystick 1 found!");
+			joystick1_nbuttons = SDL_JoystickNumButtons(joystick1);
+		}
 	}
 }
 
+void Init_Joysticks(int *argc, char *argv[])
+{
+#ifdef LPTJOY
+	char *lpt_joy0 = NULL;
+	char *lpt_joy1 = NULL;
+	int i;
+	int j;
+	
+	for (i=j=1; i<*argc; i++) {
+		if (!strcmp(argv[i], "-joy0")) {
+			if (i == *argc - 1) {
+				Aprint("joystick device path missing!");
+				break;
+			}
+			lpt_joy0 = argv[++i];
+		} else if (!strcmp(argv[i], "-joy1")) {
+			if (i == *argc - 1) {
+				Aprint("joystick device path missing!");
+				break;
+			}
+			lpt_joy1 = argv[++i];
+		} else {
+			argv[j++] = argv[i];
+		}
+	}
+	*argc = j;
+	
+	if (lpt_joy0 != NULL) {				// LPT1 joystick
+		fd_joystick0 = open(lpt_joy0, O_RDONLY);
+		if (fd_joystick0 == -1) perror(lpt_joy0);
+	}
+	if (lpt_joy1 != NULL) {				// LPT2 joystick
+		fd_joystick1 = open(lpt_joy1, O_RDONLY);
+		if (fd_joystick1 == -1) perror(lpt_joy1);
+	}
+#endif /* LPTJOY */
+	Init_SDL_Joysticks(fd_joystick0 == -1, fd_joystick1 == -1);
+}
+
+
 void Atari_Initialise(int *argc, char *argv[])
 {
-	Aprint
-		("please report SDL port bugs to Jacek Poplawski <jacekp@linux.com.pl>");
-	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) != 0) {
+	int i, j;
+	int no_joystick;
+	int width, height, bpp;
+	int help_only = FALSE;
+
+	no_joystick = 0;
+	width = ATARI_WIDTH;
+	height = ATARI_HEIGHT;
+	bpp = SDL_ATARI_BPP;
+
+	for (i = j = 1; i < *argc; i++) {
+		if (strcmp(argv[i], "-rotate90") == 0) {
+			ROTATE90 = 1;
+			width=240;
+			height=320;
+			bpp=16;
+			no_joystick = 1;
+			Aprint("rotate90 mode");
+			i++;
+		}
+		else if (strcmp(argv[i], "-nojoystick") == 0) {
+			no_joystick = 1;
+			Aprint("no joystick");
+			i++;
+		}
+		else if (strcmp(argv[i], "-width") == 0) {
+			sscanf(argv[++i], "%i", &width);
+			Aprint("width set", width);
+		}
+		else if (strcmp(argv[i], "-height") == 0) {
+			sscanf(argv[++i], "%i", &height);
+			Aprint("height set");
+		}
+		else if (strcmp(argv[i], "-bpp") == 0) {
+			sscanf(argv[++i], "%i", &bpp);
+			Aprint("bpp set");
+		}
+		else if (strcmp(argv[i], "-fullscreen") == 0) {
+			FULLSCREEN = 1;
+		}
+		else if (strcmp(argv[i], "-windowed") == 0) {
+			FULLSCREEN = 0;
+		}
+		else {
+			if (strcmp(argv[i], "-help") == 0) {
+				help_only = TRUE;
+				Aprint("\t-rotate90        Display 240x320 screen");
+				Aprint("\t-nojoystick      Disable joystick");
+#ifdef LPTJOY
+				Aprint("\t-joy0 <pathname> Select LPTjoy0 device");
+				Aprint("\t-joy1 <pathname> Select LPTjoy0 device");
+#endif /* LPTJOY */
+				Aprint("\t-width <num>     Host screen width");
+				Aprint("\t-height <num>    Host screen height");
+				Aprint("\t-bpp <num>       Host color depth");
+				Aprint("\t-fullscreen      Run fullscreen");
+				Aprint("\t-windowed        Run in window");
+			}
+			argv[j++] = argv[i];
+		}
+	}
+	*argc = j;
+
+	i = SDL_INIT_VIDEO | SDL_INIT_JOYSTICK;
+#ifdef SOUND
+	i |= SDL_INIT_AUDIO;
+#endif
+	if (SDL_Init(i) != 0) {
 		Aprint("SDL_Init FAILED");
 		Aprint(SDL_GetError());
 		Aflushlog();
 		exit(-1);
 	}
+	atexit(SDL_Quit);
 
-#if NVIDIA	/* can't set < 640x480 resolution */
-	SetNewVideoMode(672, 480, SDL_ATARI_BPP);
-#else
-	SetNewVideoMode(ATARI_WIDTH - 2 * 24, ATARI_HEIGHT, SDL_ATARI_BPP);
+	/* SDL_WM_SetIcon("/usr/local/atari800/atarixe.ICO"), NULL); */
+	SDL_WM_SetCaption(ATARI_TITLE, "Atari800");
+
+#ifdef SOUND
+	SDL_Sound_Initialise(argc, argv);
 #endif
+
+	if (help_only)
+		return;		/* return before changing the gfx mode */
+
+	SetNewVideoMode(width, height, bpp);
 	CalcPalette();
 	SetPalette();
 
+	Aprint
+		("please report SDL port bugs to Jacek Poplawski <jacekp@linux.com.pl>");
 	Aprint("video initialized");
 
-	SDL_Sound_Initialise(argc, argv);
-
-	Init_Joysticks();
+	if (no_joystick == 0)
+		Init_Joysticks(argc, argv);
 
 }
 
 int Atari_Exit(int run_monitor)
 {
 	int restart;
-	restart = FALSE;
+
+#if 0
+	if (run_monitor)
+		/* disable graphics, set alpha mode */
+		restart = monitor();
+	else
+#endif
+		restart = FALSE;
+
+	if (restart) {
+		/* set up graphics and all the stuff */
+		return 1;
+	}
 
 	SDL_Quit();
 
+	Aflushlog();
+
 	return restart;
 }
 
+void DisplayRotated240x320(Uint8 * screen)
+{
+	int i,j;
+	Uint8 c;
+	register Uint32 *start32;
+	Uint32 quad;
+	if (MainScreen->format->BitsPerPixel!=16) 
+	{
+		Aprint("rotated display works only for bpp=16 right now");
+		Aflushlog();
+		exit(-1);
+	};
+	start32 = (Uint32 *) MainScreen->pixels;
+	for (j=0;j<MainScreen->h;j++)
+	  for (i=0;i<MainScreen->w/2;i++)
+	  {
+		c=screen[ATARI_WIDTH*(i*2)+32+320-j];
+		quad=Palette16[c]<<16;
+		c=screen[ATARI_WIDTH*(i*2+1)+32+320-j];
+		quad+=Palette16[c];
+	  	*start32=quad;
+		start32++;
+	  }	
+}
+
 void DisplayWithoutScaling(Uint8 * screen, int jumped, int width)
 {
 	register Uint32 quad;
@@ -881,7 +1160,7 @@ void DisplayWithScaling(Uint8 * screen, int jumped, int width)
 	register Uint32 *start32;
 	int i;
 	int y;
-	int w2, w4;
+	int w1, w2, w4;
 	int w, h;
 	int pos;
 	int pitch4;
@@ -894,6 +1173,7 @@ void DisplayWithScaling(Uint8 * screen, int jumped, int width)
 	h = (ATARI_HEIGHT) << 16;
 	dx = w / MainScreen->w;
 	dy = h / MainScreen->h;
+	w1 = MainScreen->w - 1;
 	w2 = MainScreen->w / 2 - 1;
 	w4 = MainScreen->w / 4 - 1;
 	ss = screen;
@@ -950,7 +1230,7 @@ void DisplayWithScaling(Uint8 * screen, int jumped, int width)
 	case 32:
 		while (i > 0) {
 			x = (width + jumped) << 16;
-			pos = w2;
+			pos = w1;
 			yy = ATARI_WIDTH * (y >> 16);
 			while (pos >= 0) {
 
@@ -1001,6 +1281,11 @@ void Atari_DisplayScreen(UBYTE * screen)
 		break;
 	}
 
+	if (ROTATE90)
+	{
+		DisplayRotated240x320(screen);
+	}
+	else
 	if ((MainScreen->w == width)
 		&& (MainScreen->h == ATARI_HEIGHT)) {
 		DisplayWithoutScaling(screen, jumped, width);
@@ -1024,10 +1309,79 @@ int Atari_TRIG(int num)
 	return 0;
 }
 
+int get_SDL_joystick_state(SDL_Joystick *joystick)
+{
+	int x;
+	int y;
+
+	x = SDL_JoystickGetAxis(joystick, 0);
+	y = SDL_JoystickGetAxis(joystick, 1);
+
+	if (x > minjoy) {
+		if (y < -minjoy)
+			return STICK_UR;
+		else if (y > minjoy)
+			return STICK_LR;
+		else
+			return STICK_RIGHT;
+	} else if (x < -minjoy) {
+		if (y < -minjoy)
+			return STICK_UL;
+		else if (y > minjoy)
+			return STICK_LL;
+		else
+			return STICK_LEFT;
+	} else {
+		if (y < -minjoy)
+			return STICK_FORWARD;
+		else if (y > minjoy)
+			return STICK_BACK;
+		else
+			return STICK_CENTRE;
+	}
+}
+
+int get_LPT_joystick_state(int fd)
+{
+#ifdef LPTJOY
+	int status;
+
+	ioctl(fd, LPGETSTATUS, &status);
+	status ^= 0x78;
+
+	if (status & 0x40) {			// right
+		if (status & 0x10) {		// up
+			return STICK_UR;
+		} else if (status & 0x20) {	// down
+			return STICK_LR;
+		} else {
+			return STICK_RIGHT;
+		}
+	} else if (status & 0x80) {		// left
+		if (status & 0x10) {		// up
+			return STICK_UL;
+		} else if (status & 0x20) {	// down
+			return STICK_LL;
+		} else {
+			return STICK_LEFT;
+		}
+	} else {
+		if (status & 0x10) {		// up
+			return STICK_FORWARD;
+		} else if (status & 0x20) {	// down
+			return STICK_BACK;
+		} else {
+			return STICK_CENTRE;
+		}
+	}
+#else
+	return 0;
+#endif /* LPTJOY */
+}
+
 void SDL_Atari_PORT(Uint8 * s0, Uint8 * s1)
 {
 	int stick0, stick1;
-	int x, y;
 	stick0 = STICK_CENTRE;
 	stick1 = STICK_CENTRE;
 
@@ -1085,65 +1439,16 @@ void SDL_Atari_PORT(Uint8 * s0, Uint8 * s1)
 	{
 		SDL_JoystickUpdate();
 	}
-	if (joystick0 != NULL) {
-		x = SDL_JoystickGetAxis(joystick0, 0);
-		y = SDL_JoystickGetAxis(joystick0, 1);
-		if (x > minjoy) {
-			if (y < -minjoy)
-				stick0 = STICK_UR;
-			else if (y > minjoy)
-				stick0 = STICK_LR;
-			else
-				stick0 = STICK_RIGHT;
-		}
-		else if (x < -minjoy) {
-			if (y < -minjoy)
-				stick0 = STICK_UL;
-			else if (y > minjoy)
-				stick0 = STICK_LL;
-			else
-				stick0 = STICK_LEFT;
-		}
-		else {
-			if (y < -minjoy)
-				stick0 = STICK_FORWARD;
-			else if (y > minjoy)
-				stick0 = STICK_BACK;
-			else
-				stick0 = STICK_CENTRE;
-		}
-		*s0 = stick0;
-	}
-	if (joystick1 != NULL) {
-		x = SDL_JoystickGetAxis(joystick1, 0);
-		y = SDL_JoystickGetAxis(joystick1, 1);
-		if (x > minjoy) {
-			if (y < -minjoy)
-				stick1 = STICK_UR;
-			else if (y > minjoy)
-				stick1 = STICK_LR;
-			else
-				stick1 = STICK_RIGHT;
-		}
-		else if (x < -minjoy) {
-			if (y < -minjoy)
-				stick1 = STICK_UL;
-			else if (y > minjoy)
-				stick1 = STICK_LL;
-			else
-				stick1 = STICK_LEFT;
-		}
-		else {
-			if (y < -minjoy)
-				stick1 = STICK_FORWARD;
-			else if (y > minjoy)
-				stick1 = STICK_BACK;
-			else
-				stick1 = STICK_CENTRE;
-		}
-		*s1 = stick1;
-	}
+	
+	if (fd_joystick0 != -1)
+		*s0 = get_LPT_joystick_state(fd_joystick0);
+	else if (joystick0 != NULL)
+		*s0 = get_SDL_joystick_state(joystick0);
 
+	if (fd_joystick1 != -1)
+		*s1 = get_LPT_joystick_state(fd_joystick1);
+	else if (joystick1 != NULL)
+		*s1 = get_SDL_joystick_state(joystick1);
 }
 
 void SDL_Atari_TRIG(Uint8 * t0, Uint8 * t1)
@@ -1169,7 +1474,16 @@ void SDL_Atari_TRIG(Uint8 * t0, Uint8 * t1)
 		*t1 = trig1;
 	}
 
-	if (joystick0 != NULL) {
+	if (fd_joystick0 != -1) {
+#ifdef LPTJOY
+		int status;
+		ioctl(fd_joystick0, LPGETSTATUS, &status);
+		if (status & 8)
+			*t0 = 1;
+		else
+			*t0 = 0;
+#endif /* LPTJOY */
+	} else if (joystick0 != NULL) {
 		trig0 = 1;
 		for (i = 0; i < joystick0_nbuttons; i++) {
 			if (SDL_JoystickGetButton(joystick0, i)) {
@@ -1179,7 +1493,17 @@ void SDL_Atari_TRIG(Uint8 * t0, Uint8 * t1)
 		}
 		*t0 = trig0;
 	}
-	if (joystick1 != NULL) {
+	
+	if (fd_joystick1 != -1) {
+#ifdef LPTJOY
+		int status;
+		ioctl(fd_joystick1, LPGETSTATUS, &status);
+		if (status & 8)
+			*t1 = 1;
+		else
+			*t1 = 0;
+#endif /* LPTJOY */
+	} else if (joystick1 != NULL) {
 		trig1 = 1;
 		for (i = 0; i < joystick1_nbuttons; i++) {
 			if (SDL_JoystickGetButton(joystick1, i)) {
@@ -1214,10 +1538,10 @@ int main(int argc, char **argv)
 	if (!Atari800_Initialise(&argc, argv))
 		return 3;
 
-	refresh_rate = 1;			// ;-)  
-
+#ifdef SOUND
 	if (sound_enabled)
 		SDL_PauseAudio(0);
+#endif
 	while (!done) {
 		keycode = Atari_Keyboard();
 
@@ -1232,11 +1556,15 @@ int main(int argc, char **argv)
 			Warmstart();
 			break;
 		case AKEY_UI:
+#ifdef SOUND
 			if (sound_enabled)
 				SDL_PauseAudio(1);
+#endif
 			ui((UBYTE *) atari_screen);
+#ifdef SOUND
 			if (sound_enabled)
 				SDL_PauseAudio(0);
+#endif
 			break;
 		case AKEY_SCREENSHOT:
 			Save_PCX_file(FALSE, Find_PCX_name());
@@ -1247,24 +1575,22 @@ int main(int argc, char **argv)
 		case AKEY_BREAK:
 			key_break = 1;
 			if (!last_key_break) {
-				IRQST &= ~0x80;
 				if (IRQEN & 0x80) {
+					IRQST &= ~0x80;
 					GenerateIRQ();
 				}
 				break;
 		default:
 				key_break = 0;
-				if (keycode != SDL_JOY_0_LEFT		// plain hack
+				if (keycode != SDL_JOY_0_LEFT	// plain hack
 					&& keycode != SDL_JOY_0_RIGHT	// see atari_vga.c
-					&& keycode != SDL_JOY_0_UP		// for better way
+					&& keycode != SDL_JOY_0_UP	// for better way
 					&& keycode != SDL_JOY_0_DOWN
 					&& keycode != SDL_JOY_0_LEFTDOWN
 					&& keycode != SDL_JOY_0_RIGHTDOWN
 					&& keycode != SDL_JOY_0_LEFTUP
 					&& keycode != SDL_JOY_0_RIGHTUP
-					&& keycode != SDL_TRIG_0
-					&& keycode != SDL_TRIG_0_B
-					)
+					&& keycode != SDL_TRIG_0 && keycode != SDL_TRIG_0_B)
 					key_code = keycode;
 				break;
 			}
@@ -1282,10 +1608,17 @@ int main(int argc, char **argv)
 			if ((key_code ^ last_key_code) & ~AKEY_SHFTCTRL) {
 				last_key_code = key_code;
 				KBCODE = (UBYTE) key_code;
-				IRQST &= ~0x40;
 				if (IRQEN & 0x40) {
-					GenerateIRQ();
-				}
+					if (IRQST & 0x40) {
+						IRQST &= ~0x40;
+						GenerateIRQ();
+					}
+					else {
+						/* keyboard over-run */
+						SKSTAT &= ~0x40;
+						/* assert(IRQ != 0); */
+					}
+ 				}
 			}
 		}
 
@@ -1297,7 +1630,9 @@ int main(int argc, char **argv)
 		Device_Frame();
 		GTIA_Frame();
 		ANTIC_Frame(TRUE);
+#ifdef SHOW_DISK_LED
 		LED_Frame();
+#endif
 		POKEY_Frame();
 		nframes++;
 		atari_sync();
@@ -1313,6 +1648,72 @@ int main(int argc, char **argv)
 
 /*
  $Log: atari_sdl.c,v $
+ Revision 1.38  2003/10/25 19:07:48  joy
+ keyboard overrun fix in SDL
+
+ Revision 1.37  2003/08/05 13:22:37  joy
+ removed unused stuff
+
+ Revision 1.36  2003/03/03 09:57:32  joy
+ Ed improved autoconf again plus fixed some little things
+
+ Revision 1.35  2003/02/24 09:32:42  joy
+ header cleanup
+
+ Revision 1.34  2003/02/18 09:07:52  joy
+ more Atari keys supported: CARET, HELP, CAPSLOCK, SETTAB, CLRTAB, DELETE_LINE, DELETE_CHAR, INSERT_LINE, INSERT_CHAR
+
+ Revision 1.33  2003/02/17 14:21:33  joy
+ window title set
+
+ Revision 1.32  2003/02/09 21:22:39  joy
+ removed obsolete and unnecessary cmdline switches
+
+ Revision 1.31  2003/02/08 20:06:09  joy
+ input.c had a bug that Piotr fixed, I just copied the bugfix here.
+
+ Revision 1.30  2003/01/27 14:13:02  joy
+ Perry's cycle-exact ANTIC/GTIA emulation
+
+ Revision 1.29  2002/12/14 11:02:56  jacek_poplawski
+
+ updated audio stuff (mzpokey, stereo, 16bit audio)
+
+ Revision 1.28  2002/08/18 20:24:12  joy
+ MacOSX needs SetPalette private
+
+ Revision 1.27  2002/08/07 08:54:34  joy
+ when sound init failed Atari800 continues to run with sound disabled
+
+ Revision 1.26  2002/08/07 07:26:58  joy
+ SDL cleanup thanks to atexit, -nosound supported, -disable-SOUND supported, -dsprate added, -help fixed
+
+ Revision 1.25  2002/06/27 21:50:25  joy
+ LPTjoy in SDL works under Linux only
+
+ Revision 1.24  2002/06/27 21:20:08  joy
+ LPT joy in SDL
+
+ Revision 1.23  2002/06/23 21:52:49  joy
+ "-help" added
+ options handling fixed.
+ "--option" converted to "-option" to conform to the rest of application
+
+ Revision 1.22  2002/04/23 00:26:54  jacek_poplawski
+
+ added command line "--rotate90" for Sharp PDA
+
+ Revision 1.21  2002/04/22 23:37:17  jacek_poplawski
+
+ added command line options to SDL port
+
+ Revision 1.20  2002/02/23 05:12:36  jacek_poplawski
+ *** empty log message ***
+
+ Revision 1.19  2002/02/13 08:38:39  joy
+ fixed sound init (should have beeen unsigned = _U8)
+ screen offset in 32bpp fixed
+
  Revision 1.18  2001/12/31 08:44:53  joy
  updated
 
