@@ -1,4 +1,27 @@
-/* $Id: atari_svgalib.c,v 1.7 2001/10/03 16:15:58 knik Exp $ */
+/*
+ * atari_svgalib.c - SVGALIB library specific port code
+ *
+ * Copyright (c) 1995-1998 David Firth
+ * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -18,7 +41,7 @@
 
 extern int refresh_rate;
 
-#ifdef JOYMOUSE
+#ifdef SVGA_JOYMOUSE
 #include <vgamouse.h>
 
 #define CENTER_X 16384
@@ -95,7 +118,7 @@ static UBYTE joymask[] =
 #define FALSE 0
 #define TRUE 1
 
-#ifdef JOYMOUSE
+#ifdef SVGA_JOYMOUSE
 static int vgamouse_stick;
 static int vgamouse_strig;
 #endif
@@ -446,7 +469,7 @@ void Atari_Initialise(int *argc, char *argv[])
 		}
 		else {
 			if (strcmp(argv[i], "-help") == 0) {
-				printf("\t-interlace    Generate screen with interlace\n");
+				Aprint("\t-interlace    Generate screen with interlace\n");
 			}
 			argv[j++] = argv[i];
 		}
@@ -454,7 +477,7 @@ void Atari_Initialise(int *argc, char *argv[])
 
 	*argc = j;
 
-#ifdef JOYMOUSE
+#ifdef SVGA_JOYMOUSE
 	if (mouse_init("/dev/mouse", vga_getmousetype(), MOUSE_DEFAULTSAMPLERATE) == -1) {
 		perror("/dev/mouse");
 		exit(1);
@@ -571,7 +594,7 @@ int Atari_Exit(int run_monitor)
 		}
 	}
 	else {
-#ifdef JOYMOUSE
+#ifdef SVGA_JOYMOUSE
 		mouse_close();
 #endif
 
@@ -587,9 +610,7 @@ int Atari_Exit(int run_monitor)
 		Sound_Exit();
 #endif
 
-#ifdef BUFFERED_LOG
-        	Aflushlog();
-#endif
+      	Aflushlog();
 	}
 
 	return restart;
@@ -621,7 +642,7 @@ void Atari_DisplayScreen(UBYTE * screen)
   writepage ^= 1;
 #endif
 
-#ifdef JOYMOUSE
+#ifdef SVGA_JOYMOUSE
 	vgamouse_stick = 0xff;
 
 	if (mouse_update() != 0) {
@@ -683,7 +704,7 @@ int Atari_PORT(int num)
 	  
 //sorry, no time for this now 
 //
-//#ifdef JOYMOUSE
+//#ifdef SVGA_JOYMOUSE
 //		stick0 = vgamouse_stick;
 //#endif
 
@@ -704,7 +725,7 @@ int Atari_TRIG(int num)
 
 //sorry, no time for this now 
 //	  
-//#ifdef JOYMOUSE
+//#ifdef SVGA_JOYMOUSE
 //		trig0 = vgamouse_strig;
 //#endif
 
@@ -815,6 +836,18 @@ int main(int argc, char **argv)
 
 /*
 $Log: atari_svgalib.c,v $
+Revision 1.11  2003/03/03 09:57:32  joy
+Ed improved autoconf again plus fixed some little things
+
+Revision 1.10  2003/02/24 09:32:43  joy
+header cleanup
+
+Revision 1.9  2003/02/19 14:07:46  joy
+configure stuff cleanup
+
+Revision 1.8  2002/08/07 06:16:50  joy
+printf->Aprint
+
 Revision 1.7  2001/10/03 16:15:58  knik
 keyboard update
 
