@@ -1,3 +1,6 @@
+#ifndef _CARTRIDGE_H_
+#define _CARTRIDGE_H_
+
 #include "atari.h"
 
 #define CART_NONE		0
@@ -8,7 +11,7 @@
 #define CART_DB_32		5
 #define CART_5200_EE_16	6
 #define CART_5200_40	7
-#define CART_D50X_64	8
+#define CART_WILL_64	8
 #define CART_EXP_64		9
 #define CART_DIAMOND_64	10
 #define CART_SDX_64		11
@@ -22,9 +25,28 @@
 #define CART_5200_8		19
 #define CART_5200_4		20
 #define CART_RIGHT_8	21
-#define CART_LAST_SUPPORTED 21
+#define CART_WILL_32	22
+#define CART_XEGS_256	23
+#define CART_XEGS_512	24
+#define CART_XEGS_1024	25
+#define CART_MEGA_16	26
+#define CART_MEGA_32	27
+#define CART_MEGA_64	28
+#define CART_MEGA_128	29
+#define CART_MEGA_256	30
+#define CART_MEGA_512	31
+#define CART_MEGA_1024	32
+#define CART_SWXEGS_32	33
+#define CART_SWXEGS_64	34
+#define CART_SWXEGS_128	35
+#define CART_SWXEGS_256	36
+#define CART_SWXEGS_512	37
+#define CART_SWXEGS_1024 38
+#define CART_PHOENIX_8	39
+#define CART_BLIZZARD_16 40
+#define CART_LAST_SUPPORTED 40
 
-#define CART_MAX_SIZE	131072
+#define CART_MAX_SIZE	(1024 * 1024)
 extern int cart_kb[CART_LAST_SUPPORTED + 1];
 extern int cart_type;
 
@@ -43,3 +65,5 @@ UBYTE CART_GetByte(UWORD addr);
 void CART_PutByte(UWORD addr, UBYTE byte);
 void CART_BountyBob1(UWORD addr);
 void CART_BountyBob2(UWORD addr);
+
+#endif /* _CARTRIDGE_H_ */
