@@ -1,3 +1,6 @@
+#ifndef _CASSETTE_H_
+#define _CASSETTE_H_
+
 #include <stdio.h>		/* for FILE and FILENAME_MAX */
 
 #include "atari.h"		/* for UBYTE */
@@ -16,7 +19,14 @@ extern int cassette_current_block;
 extern int cassette_max_block;
 
 extern int hold_start;
+extern int hold_start_on_reboot; /* preserve hold_start after reboot */
 extern int press_space;
 
-UBYTE CASSETTE_Sio(void);
+int CASSETTE_AddGap(int gaptime);
+void CASSETTE_LeaderLoad();
+void CASSETTE_LeaderSave();
+int CASSETTE_Read(void);
+int CASSETTE_Write(int length);
 extern UBYTE cassette_buffer[4096];
+
+#endif /* _CASSETTE_H_ */
