@@ -1,4 +1,27 @@
-/*	CPU.C  */
+/*
+ * cpu.c - 6502 CPU emulation
+ *
+ * Copyright (C) 1995-1998 David Firth
+ * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 /*
 	Compilation options
 	===================
@@ -64,7 +87,6 @@
 
 5.	Make X and Y registers UWORD or unsigned int, so they don't need to
 	be zero-extended while computing address in indexed mode.
-
  */
 
 #include <stdio.h>
@@ -79,7 +101,7 @@
 #include "statesav.h"
 #include "ui.h"
 
-#ifdef CPUASS
+#ifdef FALCON_CPUASM
 extern UBYTE IRQ;
 
 #ifdef PAGED_MEM
@@ -115,15 +137,29 @@ void CPU_PutStatus(void)
 #define PL dGetByte(0x0100 + ++S)
 #define PH(x) dPutByte(0x0100 + S--, x)
 
-#ifndef NO_CYCLE_EXACT
-#ifdef PAGED_MEM
-#define RMW_GetByte(x,addr) x = GetByte(addr); if ((addr & 0xef1f) == 0xc01a) { xpos--; PutByte(addr,x); xpos++; }
+#ifdef CYCLE_EXACT
+#ifndef PAGED_ATTRIB
+#define RMW_GetByte(x,addr) \
+		if (attrib[addr] == HARDWARE) { \
+			x = Atari800_GetByte(addr); \
+			if ((addr & 0xef1f) == 0xc01a) { \
+				 xpos--; \
+				 Atari800_PutByte(addr, x); \
+				 xpos++; \
+			} \
+		} else x = memory[addr];
 #else
-#define RMW_GetByte(x,addr)	if (attrib[addr] == HARDWARE) { x = Atari800_GetByte(addr); if ((addr & 0xef1f) == 0xc01a) { xpos--; Atari800_PutByte(addr, x); xpos++; }} else x = memory[addr];
-#endif /* PAGED_MEM */
+#define RMW_GetByte(x,addr) \
+		x = GetByte(addr); \
+		if ((addr & 0xef1f) == 0xc01a) { \
+			xpos--; \
+			PutByte(addr,x); \
+			xpos++; \
+		}
+#endif /* PAGED_ATTRIB */
 #else
 #define RMW_GetByte(x,addr) x = GetByte(addr);
-#endif /* NO_CYCLE_EXACT */
+#endif /* CYCLE_EXACT */
 
 #define PHW(x) PH((x)>>8); PH((x) & 0xff)
 
@@ -158,8 +194,15 @@ UBYTE IRQ;
 
 #ifdef MONITOR_BREAK
 UWORD remember_PC[REMEMBER_PC_STEPS];
+int remember_PC_curpos = 0;
+#ifdef NEW_CYCLE_EXACT
+int remember_xpos[REMEMBER_PC_STEPS];
+int remember_xpos_curpos = 0;
+#endif
+extern UWORD ypos_break_addr;
 extern UWORD break_addr;
 UWORD remember_JMP[REMEMBER_JMP_STEPS];
+int remember_jmp_curpos=0;
 extern UBYTE break_step;
 extern UBYTE break_ret;
 extern UBYTE break_cim;
@@ -267,31 +310,27 @@ void NMI(void)
 
 /* check pending IRQ, helps in (not only) Lucasfilm games */
 #ifdef MONITOR_BREAK
-#define CPUCHECKIRQ							\
-{									\
-	if (IRQ) {							\
-		if (!(regP & I_FLAG)) {					\
-			PHW(PC);					\
-			PHPB0;						\
-			SetI;						\
-			PC = dGetWordAligned(0xfffe);				\
-			xpos += 7;					\
-			ret_nesting+=1;					\
-		}							\
-	}								\
+#define CPUCHECKIRQ \
+{ \
+	if (IRQ && !(regP & I_FLAG) && xpos < xpos_limit) { \
+		PHW(PC); \
+		PHPB0; \
+		SetI; \
+		PC = dGetWordAligned(0xfffe); \
+		xpos += 7; \
+		ret_nesting+=1; \
+	} \
 }
 #else
-#define CPUCHECKIRQ							\
-{									\
-	if (IRQ) {							\
-		if (!(regP & I_FLAG)) {					\
-			PHW(PC);					\
-			PHPB0;						\
-			SetI;						\
-			PC = dGetWordAligned(0xfffe);				\
-			xpos += 7;					\
-		}							\
-	}								\
+#define CPUCHECKIRQ \
+{ \
+	if (IRQ && !(regP & I_FLAG) && xpos < xpos_limit) { \
+		PHW(PC); \
+		PHPB0; \
+		SetI; \
+		PC = dGetWordAligned(0xfffe); \
+		xpos += 7; \
+	} \
 }
 #endif
 
@@ -449,13 +488,36 @@ void GO(int limit)
    2. The timing of the IRQs are not that critical.
  */
 
+#ifdef NEW_CYCLE_EXACT
+	if (wsync_halt) {
+		if(DRAWING_SCREEN){
+/* if WSYNC_C is a stolen cycle, antic2cpu_ptr will convert that to the nearest
+   cpu cycle before that cycle.  The CPU will see this cycle, if WSYNC is not 
+   delayed. (Acutally this cycle is the first cycle of the instruction after
+  STA WSYNC, which was really executed one cycle after STA WSYNC because
+  of an internal antic delay ).   delayed_wsync is added to this cycle to form
+   the limit in the case that WSYNC is not early (does not allow this extra cycle) */
+ 
+			if (limit<antic2cpu_ptr[WSYNC_C]+delayed_wsync)
+				return;
+			xpos = antic2cpu_ptr[WSYNC_C]+delayed_wsync;
+		}else{
+			if (limit<(WSYNC_C+delayed_wsync))
+				return;
+			xpos = WSYNC_C;
+			
+		}
+		wsync_halt = 0;
+		delayed_wsync=0;
+	}
+#else
 	if (wsync_halt) {
 		if (limit<WSYNC_C)
 			return;
 		xpos = WSYNC_C;
 		wsync_halt = 0;
 	}
-
+#endif /*NEW_CYCLE_EXACT*/
 	xpos_limit = limit;			/* needed for WSYNC store inside ANTIC */
 
 	UPDATE_LOCAL_REGS;
@@ -492,10 +554,19 @@ void GO(int limit)
 #endif
 
 #ifdef MONITOR_BREAK
-		memmove(&remember_PC[0], &remember_PC[1], (REMEMBER_PC_STEPS - 1) * 2);
-		remember_PC[REMEMBER_PC_STEPS - 1] = PC;
+		remember_PC[remember_PC_curpos]=PC;
+		remember_PC_curpos=(remember_PC_curpos+1)%REMEMBER_PC_STEPS;
+#ifdef NEW_CYCLE_EXACT
+		if(DRAWING_SCREEN){
+			remember_xpos[remember_xpos_curpos] = cpu2antic_ptr[xpos]+(ypos<<8);
+		}else{
+			remember_xpos[remember_xpos_curpos] = xpos+(ypos<<8);
+		}
+		remember_xpos_curpos=(remember_xpos_curpos+1)%REMEMBER_PC_STEPS;
+			
+#endif
 
-		if (break_addr == PC) {
+		if (break_addr == PC || ypos_break_addr == ypos) {
 			UPDATE_GLOBAL_REGS;
 			CPU_GetStatus();
 			if (!Atari800_Exit(TRUE))
@@ -712,8 +783,8 @@ void GO(int limit)
 		{
 			UWORD retadr = PC + 1;
 #ifdef MONITOR_BREAK
-			memmove(&remember_JMP[0], &remember_JMP[1], 2 * (REMEMBER_JMP_STEPS - 1));
-			remember_JMP[REMEMBER_JMP_STEPS - 1] = PC - 1;
+			remember_JMP[remember_jmp_curpos]=PC-1;
+			remember_jmp_curpos=(remember_jmp_curpos+1)%REMEMBER_JMP_STEPS;
 			ret_nesting++;
 #endif
 			PHW(retadr);
@@ -955,8 +1026,8 @@ void GO(int limit)
 
 	OPCODE(4c)				/* JMP abcd */
 #ifdef MONITOR_BREAK
-		memmove(&remember_JMP[0], &remember_JMP[1], 2 * (REMEMBER_JMP_STEPS - 1));
-		remember_JMP[REMEMBER_JMP_STEPS - 1] = PC - 1;
+		remember_JMP[remember_jmp_curpos]=PC-1;
+		remember_jmp_curpos=(remember_jmp_curpos+1)%REMEMBER_JMP_STEPS;
 #endif
 		PC = dGetWord(PC);
 		DONE
@@ -968,7 +1039,7 @@ void GO(int limit)
 
 	OPCODE(4e)				/* LSR abcd */
 		ABSOLUTE;
-		data = GetByte(addr);
+		RMW_GetByte(data, addr);
 		C = data & 1;
 		Z = data >> 1;
 		N = 0;
@@ -1148,8 +1219,8 @@ void GO(int limit)
 
 	OPCODE(6c)				/* JMP (abcd) */
 #ifdef MONITOR_BREAK
-		memmove(&remember_JMP[0], &remember_JMP[1], 2 * (REMEMBER_JMP_STEPS - 1));
-		remember_JMP[REMEMBER_JMP_STEPS - 1] = PC - 1;
+		remember_JMP[remember_jmp_curpos]=PC-1;
+		remember_jmp_curpos=(remember_jmp_curpos+1)%REMEMBER_JMP_STEPS;
 #endif
 		addr = dGetWord(PC);
 #ifdef CPU65C02
@@ -1988,7 +2059,7 @@ void GO(int limit)
 void CPU_Initialise(void)
 {
 }
-#endif /* CPU_ASM */
+#endif /* FALCON_CPUASM */
 
 void CPU_Reset(void)
 {
