@@ -1,15 +1,27 @@
-/* $Id: atari_vga.c,v 1.1 2001/10/26 05:36:48 fox Exp $ */
-/* -------------------------------------------------------------------------- */
-
 /*
- * DJGPP - VGA Backend for David Firth's Atari 800 emulator
+ * atari_vga.c - DOS VGA mode specific port code
  *
- * by Ivo van Poorten (C)1996  See file COPYRIGHT for policy.
+ * Copyright (c) 1996 Ivo van Poorten
+ * Copyright (c) 1996-2003 Atari800 development team (see DOC/CREDITS)
  *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* -------------------------------------------------------------------------- */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -28,9 +40,9 @@
 #include "dos/sound_dos.h"
 #include "monitor.h"
 #include "pcjoy.h"
-#include "diskled.h"	/* for led_status */
 #include "ataripcx.h"
 #include "rt-config.h"	/* for refresh_rate */
+#include "diskled.h"	/* for led_status */
 
 #include "dos/vga_gfx.h"
 
@@ -41,6 +53,7 @@
 #define KEYBOARD_EXCLUSIVE
 */
 
+
 static int trig0;
 static int stick0;
 static int trig1;
@@ -80,6 +93,7 @@ static int use_vret=0;           /*control vertical retrace?*/
   static int vesa_selector;  /*selector for accessing LFB*/
 
 static int vga_started = 0;             /*AAA needed for DOS to see text */
+static int keyboard_handler_replaced = FALSE;
 
 extern int SHIFT;
 extern int ATKEYPRESSED;
@@ -199,18 +213,29 @@ void read_LPTjoy(int port, int joyport)
         }
 }
 
-void update_leds(void)
+static void update_leds(void)
 {
 	outportb(0x60,0xed);
-	asm("	   jmp 0f
-		0: jmp 1f
-		1:
-	");
+	asm("nop; nop");
 	outportb(0x60,	(PC_keyboard ? 0 : 2)
+#ifdef SHOW_DISK_LED
 			|(led_status ? 4 : 0)
+#endif
 			|(joy_keyboard ? 1 : 0));
 }
 
+static void update_disk_led()
+{
+#ifdef SHOW_DISK_LED
+	static int last_led_status = 0;
+
+	if (led_status != last_led_status) {
+		update_leds();
+		last_led_status = led_status;
+	}
+#endif
+}
+
 /* -------------------------------------------------------------------------- */
 /* KEYBOARD HANDLER                                                           */
 /* -------------------------------------------------------------------------- */
@@ -390,30 +415,30 @@ void key_handler(void)
 
 void key_init(void)
 {
-        int i;
-        for (i=0;i<256;i++) keypush[i]=0; /*none key is pressed*/
-        extended_key_follows=FALSE;
+	int i;
+	for (i=0;i<256;i++) keypush[i]=0; /*none key is pressed*/
+	extended_key_follows=FALSE;
 	update_leds();
-        raw_key_r=0;raw_key=0;
-        new_key_handler.pm_offset = (int) key_handler;
-        new_key_handler.pm_selector = _go32_my_cs();
-        _go32_dpmi_get_protected_mode_interrupt_vector(0x9, &old_key_handler);
-        _go32_dpmi_allocate_iret_wrapper(&new_key_handler);
-        _go32_dpmi_set_protected_mode_interrupt_vector(0x9, &new_key_handler);
+	raw_key_r=0;raw_key=0;
+	new_key_handler.pm_offset = (int) key_handler;
+	new_key_handler.pm_selector = _go32_my_cs();
+	_go32_dpmi_get_protected_mode_interrupt_vector(0x9, &old_key_handler);
+	_go32_dpmi_allocate_iret_wrapper(&new_key_handler);
+	_go32_dpmi_set_protected_mode_interrupt_vector(0x9, &new_key_handler);
+	keyboard_handler_replaced = TRUE;
 }
 
 void key_delete(void)
 {
-        int kflags;
-        _go32_dpmi_set_protected_mode_interrupt_vector(0x9, &old_key_handler);
-        /*set the original keyboard LED settings*/
-        kflags=_farpeekb(_dos_ds,0x417);
-        outportb(0x60,0xed);
-        asm("   jmp 0f
-             0: jmp 1f
-             1:
-             ");
-        outportb(0x60,((kflags>>4)&0x7));
+        if (keyboard_handler_replaced) {
+                int kflags;
+                _go32_dpmi_set_protected_mode_interrupt_vector(0x9, &old_key_handler);
+                /*set the original keyboard LED settings*/
+                kflags=_farpeekb(_dos_ds,0x417);
+                outportb(0x60,0xed);
+                asm("nop; nop");
+                outportb(0x60,((kflags>>4)&0x7));
+        }
 }
 
 
@@ -484,7 +509,7 @@ void SetupVgaEnvironment()
 		rg.x.ax = 0;
 		int86(0x33, &rg, &rg);
 		if (rg.x.ax != 0xffff) {
-			printf("Can't find mouse!\n");
+			Aprint("Can't find mouse!");
 			mouse_mode = MOUSE_OFF;
 		}
 	}
@@ -588,6 +613,7 @@ void Atari_Initialise(int *argc, char *argv[])
         int i;
         int j;
         int use_lpt1=0,use_lpt2=0,use_lpt3=0;
+        int help_only = FALSE;
 
         for (i = j = 1; i < *argc; i++) {
                 if (strcmp(argv[i], "-interlace") == 0) {
@@ -611,8 +637,7 @@ void Atari_Initialise(int *argc, char *argv[])
                   video_mode=atoi(argv[i]);
                   if (video_mode<0 || video_mode>3)
                   {
-                    printf("Invalid video mode, using default.\n");
-                    getchar();
+                    Aprint("Invalid video mode, using default.");
                     video_mode=0;
                   }
                 }
@@ -634,22 +659,20 @@ void Atari_Initialise(int *argc, char *argv[])
 		}
                 else {
                         if (strcmp(argv[i], "-help") == 0) {
-                                printf("\t-interlace    Generate screen with interlace\n");
-                                printf("\t-LPTjoy1      Read joystick connected to LPT1\n");
-                                printf("\t-LPTjoy2      Read joystick connected to LPT2\n");
-                                printf("\t-LPTjoy3      Read joystick connected to LPT3\n");
-                                printf("\t-joyswap      Swap joysticks\n");
-                                printf("\t-video x      Set video mode:\n");
-                                printf("\t\t0 - 320x200\n\t\t1 - 320x240\n");
-                                printf("\t\t2 - 320x240, interlaced with black lines\n");
-                                printf("\t\t3 - 320x240, interlaced with darker lines (slower!)\n");
-                                printf("\t-novesa       Do not use vesa2 videomodes\n");
-                                printf("\t-vretrace     Use vertical retrace control\n");
-				printf("\t-keyboard 0   PC keyboard layout\n");
-				printf("\t-keyboard 1   Atari keyboard layout\n");
-                                printf("\nPress Return/Enter to continue...");
-                                getchar();
-                                printf("\r                                 \n");
+                                help_only = TRUE;
+                                Aprint("\t-interlace       Generate screen with interlace");
+                                Aprint("\t-LPTjoy1         Read joystick connected to LPT1");
+                                Aprint("\t-LPTjoy2         Read joystick connected to LPT2");
+                                Aprint("\t-LPTjoy3         Read joystick connected to LPT3");
+                                Aprint("\t-joyswap         Swap joysticks");
+                                Aprint("\t-video x         Set video mode:");
+                                Aprint("\t\t0 - 320x200\n\t\t1 - 320x240");
+                                Aprint("\t\t2 - 320x240, interlaced with black lines");
+                                Aprint("\t\t3 - 320x240, interlaced with darker lines (slower!)");
+                                Aprint("\t-novesa          Do not use vesa2 videomodes");
+                                Aprint("\t-vretrace        Use vertical retrace control");
+                                Aprint("\t-keyboard 0      PC keyboard layout");
+                                Aprint("\t-keyboard 1      Atari keyboard layout");
                         }
                         argv[j++] = argv[i];
                 }
@@ -662,18 +685,19 @@ void Atari_Initialise(int *argc, char *argv[])
         Sound_Initialise(argc, argv);
 #endif
 
+        if (help_only)
+                return;
+
         /* check if joystick is connected */
-        printf("Joystick is checked...");
+        printf("Joystick is checked...\n");
         fflush(stdout);
         outportb(0x201, 0xff);
         usleep(100000UL);
         joy_in = ((inportb(0x201) & 3) == 0);
         if (joy_in)
                 joystick0(&js0_centre_x, &js0_centre_y);
-        if (joy_in)
-                printf(" found!\n");
-        else
-                printf("\n\nSorry, I see no joystick. Use numeric pad\n");
+        if (! joy_in)
+                Aprint("Sorry, I see no joystick. Use numeric pad");
 
         /*find number of VESA2 video mode*/
         if (use_vesa)
@@ -848,29 +872,25 @@ int Atari_Exit(int run_monitor)
 
         if (run_monitor) {
 #ifdef SOUND
-		Sound_Pause();
+                Sound_Pause();
 #endif
                 if (monitor()) {
 #ifdef MONITOR_BREAK
                         if (!break_step)       /*do not enter videomode when stepping through the code*/
-                          SetupVgaEnvironment();
-#else
-                        SetupVgaEnvironment();
 #endif
+                        SetupVgaEnvironment();
 #ifdef SOUND
-			Sound_Continue();
+                        Sound_Continue();
 #endif
                         return 1;                       /* return to emulation */
                 }
-	}
+        }
 
 #ifdef SOUND
         Sound_Exit();
 #endif
 
-#ifdef BUFFERED_LOG
         Aflushlog();
-#endif
 
         return 0;
 }
@@ -1450,17 +1470,14 @@ int Atari_TRIG(int num)
 
 int main(int argc, char **argv)
 {
-	int test_val = 0;
-	int last_led_status = 0;
-	int keycode;
-
 	/* initialise Atari800 core */
 	if (!Atari800_Initialise(&argc, argv))
 		return 3;
 
 	/* main loop */
 	while (TRUE) {
-		keycode = Atari_Keyboard();
+		int refresh_counter = 0;
+		int keycode = Atari_Keyboard();
 
 		switch (keycode) {
 		case AKEY_COLDSTART:
@@ -1507,17 +1524,14 @@ int main(int argc, char **argv)
 			mouse_buttons = rg.x.bx;
 		}
 
-		if (++test_val == refresh_rate) {
+		if (++refresh_counter == refresh_rate) {
 			Atari800_Frame(EMULATE_FULL);
 #ifndef DONT_SYNC_WITH_HOST
 			atari_sync(); /* here seems to be the best place to sync */
 #endif
-			if (led_status != last_led_status) {
-				update_leds();
-				last_led_status = led_status;
-			}
+			update_disk_led();
 			Atari_DisplayScreen((UBYTE *) atari_screen);
-			test_val = 0;
+			refresh_counter = 0;
 		}
 		else {
 #ifdef VERY_SLOW
