@@ -1,10 +1,9 @@
-
-#ifndef _sound_dos_h
-#define _sound_dos_h
+#ifndef _SOUND_DOS_H_
+#define _SOUND_DOS_H_
 
 void Sound_Initialise(int *argc, char *argv[]);
 void Sound_Pause(void);
 void Sound_Continue(void);
 void Sound_Exit(void);
 
-#endif /* _sound_dos_h */
+#endif /* _SOUND_DOS_H_ */
