@@ -1,98 +1,97 @@
-/* $Id: pokeysnd.c,v 1.8 2001/07/22 08:24:47 knik Exp $ */
-/*****************************************************************************/
-/*                                                                           */
-/* Module:  POKEY Chip Emulator, V2.4                                        */
-/* Purpose: To emulate the sound generation hardware of the Atari POKEY chip. */
-/* Author:  Ron Fries                                                        */
-/*                                                                           */
-/* Revision History:                                                         */
-/*                                                                           */
-/* 09/22/96 - Ron Fries - Initial Release                                    */
-/* 01/14/97 - Ron Fries - Corrected a minor problem to improve sound quality */
-/*                        Also changed names from POKEY11.x to POKEY.x       */
-/* 01/17/97 - Ron Fries - Added support for multiple POKEY chips.            */
-/* 03/31/97 - Ron Fries - Made some minor mods for MAME (changed to signed   */
-/*                        8-bit sample, increased gain range, removed        */
-/*                        _disable() and _enable().)                         */
-/* 04/06/97 - Brad Oliver - Some cross-platform modifications. Added         */
-/*                          big/little endian #defines, removed <dos.h>,     */
-/*                          conditional defines for TRUE/FALSE               */
-/* 01/19/98 - Ron Fries - Changed signed/unsigned sample support to a        */
-/*                        compile-time option.  Defaults to unsigned -       */
-/*                        define SIGNED_SAMPLES to create signed.            */
-/* 03/22/98 - Ron Fries - Added 'filter' support to channels 1 & 2.          */
-/* 09/29/99 - Krzysztof Nikiel - Changed Pokey_process main loop;            */
-/*                               added output interpolation to reduce        */
-/*                               DSP interference                            */
-/*                                                                           */
-/* V2.0 Detailed Changes                                                     */
-/* ---------------------                                                     */
-/*                                                                           */
-/* Now maintains both a POLY9 and POLY17 counter.  Though this slows the     */
-/* emulator in general, it was required to support mutiple POKEYs since      */
-/* each chip can individually select POLY9 or POLY17 operation.  Also,       */
-/* eliminated the Poly17_size variable.                                      */
-/*                                                                           */
-/* Changed address of POKEY chip.  In the original, the chip was fixed at    */
-/* location D200 for compatibility with the Atari 800 line of 8-bit          */
-/* computers. The update function now only examines the lower four bits, so  */
-/* the location for all emulated chips is effectively xxx0 - xxx8.           */
-/*                                                                           */
-/* The Update_pokey_sound function has two additional parameters which       */
-/* selects the desired chip and selects the desired gain.                    */
-/*                                                                           */
-/* Added clipping to reduce distortion, configurable at compile-time.        */
-/*                                                                           */
-/* The Pokey_sound_init function has an additional parameter which selects   */
-/* the number of pokey chips to emulate.                                     */
-/*                                                                           */
-/* The output will be amplified by gain/16.  If the output exceeds the       */
-/* maximum value after the gain, it will be limited to reduce distortion.    */
-/* The best value for the gain depends on the number of POKEYs emulated      */
-/* and the maximum volume used.  The maximum possible output for each        */
-/* channel is 15, making the maximum possible output for a single chip to    */
-/* be 60.  Assuming all four channels on the chip are used at full volume,   */
-/* a gain of 64 can be used without distortion.  If 4 POKEY chips are        */
-/* emulated and all 16 channels are used at full volume, the gain must be    */
-/* no more than 16 to prevent distortion.  Of course, if only a few of the   */
-/* 16 channels are used or not all channels are used at full volume, a       */
-/* larger gain can be used.                                                  */
-/*                                                                           */
-/* The Pokey_process routine automatically processes and mixes all selected  */
-/* chips/channels.  No additional calls or functions are required.           */
-/*                                                                           */
-/* The unoptimized Pokey_process2() function has been removed.               */
-/*                                                                           */
-/*****************************************************************************/
-/*                                                                           */
-/*                 License Information and Copyright Notice                  */
-/*                 ========================================                  */
-/*                                                                           */
-/* PokeySound is Copyright(c) 1996-1998 by Ron Fries                         */
-/*                                                                           */
-/* This library is free software; you can redistribute it and/or modify it   */
-/* under the terms of version 2 of the GNU Library General Public License    */
-/* as published by the Free Software Foundation.                             */
-/*                                                                           */
-/* This library is distributed in the hope that it will be useful, but       */
-/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library */
-/* General Public License for more details.                                  */
-/* To obtain a copy of the GNU Library General Public License, write to the  */
-/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   */
-/*                                                                           */
-/* Any permitted reproduction of these routines, in whole or in part, must   */
-/* bear this legend.                                                         */
-/*                                                                           */
-/*****************************************************************************/
+/*
+ * pokeysnd.c - POKEY sound chip emulation, v2.4
+ *
+ * Copyright (C) 1996-1998 Ron Fries
+ * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
+/*
+   Revision History:
+
+   09/22/96 - Ron Fries - Initial Release
+   01/14/97 - Ron Fries - Corrected a minor problem to improve sound quality
+                          Also changed names from POKEY11.x to POKEY.x
+   01/17/97 - Ron Fries - Added support for multiple POKEY chips.
+   03/31/97 - Ron Fries - Made some minor mods for MAME (changed to signed
+                          8-bit sample, increased gain range, removed
+                          _disable() and _enable().)
+   04/06/97 - Brad Oliver - Some cross-platform modifications. Added
+                            big/little endian #defines, removed <dos.h>,
+                            conditional defines for TRUE/FALSE
+   01/19/98 - Ron Fries - Changed signed/unsigned sample support to a
+                          compile-time option.  Defaults to unsigned -
+                          define SIGNED_SAMPLES to create signed.
+   03/22/98 - Ron Fries - Added 'filter' support to channels 1 & 2.
+   09/29/99 - Krzysztof Nikiel - Changed Pokey_process main loop;
+                                 added output interpolation to reduce
+                                 DSP interference
+
+   V2.0 Detailed Changes
+   ---------------------
+
+   Now maintains both a POLY9 and POLY17 counter.  Though this slows the
+   emulator in general, it was required to support mutiple POKEYs since
+   each chip can individually select POLY9 or POLY17 operation.  Also,
+   eliminated the Poly17_size variable.
+
+   Changed address of POKEY chip.  In the original, the chip was fixed at
+   location D200 for compatibility with the Atari 800 line of 8-bit
+   computers. The update function now only examines the lower four bits, so
+   the location for all emulated chips is effectively xxx0 - xxx8.
+
+   The Update_pokey_sound function has two additional parameters which
+   selects the desired chip and selects the desired gain.
+
+   Added clipping to reduce distortion, configurable at compile-time.
+
+   The Pokey_sound_init function has an additional parameter which selects
+   the number of pokey chips to emulate.
+
+   The output will be amplified by gain/16.  If the output exceeds the
+   maximum value after the gain, it will be limited to reduce distortion.
+   The best value for the gain depends on the number of POKEYs emulated
+   and the maximum volume used.  The maximum possible output for each
+   channel is 15, making the maximum possible output for a single chip to
+   be 60.  Assuming all four channels on the chip are used at full volume,
+   a gain of 64 can be used without distortion.  If 4 POKEY chips are
+   emulated and all 16 channels are used at full volume, the gain must be
+   no more than 16 to prevent distortion.  Of course, if only a few of the
+   16 channels are used or not all channels are used at full volume, a
+   larger gain can be used.
+
+   The Pokey_process routine automatically processes and mixes all selected
+   chips/channels.  No additional calls or functions are required.
+
+   The unoptimized Pokey_process2() function has been removed.
+*/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
+#include <limits.h>
 
 #include "pokeysnd.h"
+#include "mzpokeysnd.h"
 #include "atari.h"
 #include "sndsave.h"
+#include "rt-config.h"	/* extern for stereo_enabled and enable_new_pokey */
 
 #include "config.h"
 
@@ -170,18 +169,15 @@ static uint32 Samp_n_max,		/* Sample max.  For accuracy, it is *256 */
 
 extern int atari_speaker;
 
-#ifndef NOSNDINTER
+#ifdef INTERPOLATE_SOUND
 static uint16 last_val = 0;		/* last output value */
+#ifdef STEREO_SOUND
+static uint16 last_val2 = 0;	/* last output value */
 #endif
-#ifdef STEREO
-#ifndef NOSNDINTER
-static uint16 last_val2 = 0;		/* last output value */
-#endif
-extern int stereo_enabled;
 #endif
 
 /* Volume only emulations declarations */
-#ifndef	NO_VOL_ONLY
+#ifdef VOL_ONLY_SOUND
 
 #define	SAMPBUF_MAX	2000
 int	sampbuf_val[SAMPBUF_MAX];	/* volume values */
@@ -194,7 +190,7 @@ int	sampbuf_lastval = 0;		/* last volume */
 int	sampout;			/* last out volume */
 uint16	samp_freq;
 int	samp_consol_val=0;		/* actual value of console sound */
-#ifdef STEREO
+#ifdef STEREO_SOUND
 int	sampbuf_val2[SAMPBUF_MAX];	/* volume values */
 int	sampbuf_cnt2[SAMPBUF_MAX];	/* relative start time */
 int	sampbuf_ptr2 = 0;		/* pointer to sampbuf */
@@ -203,7 +199,40 @@ int	sampbuf_last2 = 0;		/* last absolute time */
 int	sampbuf_lastval2 = 0;		/* last volume */
 int	sampout2;			/* last out volume */
 #endif
-#endif	/* NO_VOL_ONLY */
+#endif  /* VOL_ONLY_SOUND */
+
+static uint32 snd_freq17 = FREQ_17_EXACT;
+static uint16 snd_playback_freq = 44100;
+static uint8 snd_num_pokeys = 1;
+static int snd_flags = 0;
+static int mz_quality = 0;		/* default quality for mzpokeysnd */
+
+/* multiple sound engine interface */
+static void Pokey_process_8(void * sndbuffer, unsigned sndn);
+static void Pokey_process_16(void * sndbuffer, unsigned sndn);
+static void null_pokey_process(void *sndbuffer, unsigned int sndn) {}
+void (* Pokey_process)(void * sndbuffer, unsigned int sndn) = null_pokey_process;
+
+static void Update_pokey_sound_rf(uint16, uint8, uint8, uint8);
+static void null_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain) {}
+void (*Update_pokey_sound) (uint16 addr, uint8 val, uint8 chip, uint8 gain)
+  = null_pokey_sound;
+
+#ifdef SERIO_SOUND
+static void Update_serio_sound_rf(int out, UBYTE data);
+static void null_serio_sound(int out, UBYTE data) {}
+void (*Update_serio_sound)(int out, UBYTE data) = null_serio_sound;
+#endif
+int serio_sound_enabled = 1;
+
+static void Update_consol_sound_rf(int set);
+static void null_consol_sound(int set) {}
+void (*Update_consol_sound)(int set) = null_consol_sound;
+int console_sound_enabled = 1;
+
+static void Update_vol_only_sound_rf(void);
+static void null_vol_only_sound(void) {}
+void (*Update_vol_only_sound)(void) = null_vol_only_sound;
 
 /*****************************************************************************/
 /* In my routines, I treat the sample output as another divide by N counter  */
@@ -248,13 +277,59 @@ int	sampout2;			/* last out volume */
 /*                                                                           */
 /*****************************************************************************/
 
-void Pokey_sound_init(uint32 freq17, uint16 playback_freq, uint8 num_pokeys)
+static int Pokey_sound_init_rf(uint32 freq17, uint16 playback_freq,
+           uint8 num_pokeys, unsigned int flags);
+
+int Pokey_DoInit()
+{
+	if (enable_new_pokey)
+		return Pokey_sound_init_mz(snd_freq17, snd_playback_freq,
+				snd_num_pokeys, snd_flags, mz_quality);
+	else
+		return Pokey_sound_init_rf(snd_freq17, snd_playback_freq,
+				snd_num_pokeys, snd_flags);
+}
+
+int Pokey_sound_init(uint32 freq17, uint16 playback_freq, uint8 num_pokeys,
+		     unsigned int flags)
+{
+	snd_freq17 = freq17;
+	snd_playback_freq = playback_freq;
+	snd_num_pokeys = num_pokeys;
+	snd_flags = flags;
+
+	return Pokey_DoInit();
+}
+
+void Pokey_set_mzquality(int quality)	/* specially for win32, perhaps not needed? */
+{
+	mz_quality = quality;
+}
+
+static int Pokey_sound_init_rf(uint32 freq17, uint16 playback_freq,
+           uint8 num_pokeys, unsigned int flags)
 {
 	uint8 chan;
 
-#ifndef	NO_VOL_ONLY
+	Update_pokey_sound = Update_pokey_sound_rf;
+#ifdef SERIO_SOUND
+	Update_serio_sound = Update_serio_sound_rf;
+#endif
+#ifdef CONSOLE_SOUND
+	Update_consol_sound = Update_consol_sound_rf;
+#endif
+#ifdef VOL_ONLY_SOUND
+	Update_vol_only_sound = Update_vol_only_sound_rf;
+#endif
+
+	if (flags & SND_BIT16)
+	  Pokey_process = Pokey_process_16;
+        else
+	  Pokey_process = Pokey_process_8;
+
+#ifdef VOL_ONLY_SOUND
 	samp_freq=playback_freq;
-#endif	/* NO_VOL_ONLY */
+#endif  /* VOL_ONLY_SOUND */
 
 	/* disable interrupts to handle critical sections */
 	/*    _disable(); */ /* RSF - removed for portability 31-MAR-97 */
@@ -277,7 +352,7 @@ void Pokey_sound_init(uint32 freq17, uint16 playback_freq, uint8 num_pokeys)
 		Div_n_cnt[chan] = 0;
 		Div_n_max[chan] = 0x7fffffffL;
 		AUDV[chan] = 0;
-#ifndef	NO_VOL_ONLY
+#ifdef VOL_ONLY_SOUND
 		sampbuf_AUDV[chan] = 0;
 #endif
 	}
@@ -286,6 +361,8 @@ void Pokey_sound_init(uint32 freq17, uint16 playback_freq, uint8 num_pokeys)
 	Num_pokeys = num_pokeys;
 
 	/*    _enable(); */ /* RSF - removed for portability 31-MAR-97 */
+
+	return 0; // OK
 }
 
 
@@ -308,7 +385,8 @@ void Pokey_sound_init(uint32 freq17, uint16 playback_freq, uint8 num_pokeys)
 /*                                                                           */
 /*****************************************************************************/
 
-void Update_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain)
+static void Update_pokey_sound_rf(uint16 addr, uint8 val, uint8 chip,
+				  uint8 gain)
 {
 	uint32 new_val = 0;
 	uint8 chan;
@@ -477,10 +555,10 @@ void Update_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain)
 	/* if channel is volume only, set current output */
 	for (chan = CHAN1; chan <= CHAN4; chan++) {
 		if (chan_mask & (1 << chan)) {
-#ifndef	NO_VOL_ONLY
+#ifdef VOL_ONLY_SOUND
 			if( (AUDC[chan + chip_offs] & VOL_ONLY) )
 			{
-#ifdef STEREO
+#ifdef STEREO_SOUND
 				if( chip&0x01 )
 				{
 				sampbuf_lastval2+=AUDV[chan + chip_offs]
@@ -519,11 +597,11 @@ void Update_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain)
 					if( sampbuf_rptr>=SAMPBUF_MAX )
 						sampbuf_rptr=0;
 				}
-#ifdef STEREO
+#ifdef STEREO_SOUND
 				}
 #endif
 			}
-#endif	/* NO_VOL_ONLY */
+#endif  /* VOL_ONLY_SOUND */
 			/* I've disabled any frequencies that exceed the sampling
 			   frequency.  There isn't much point in processing frequencies
 			   that the hardware can't reproduce.  I've also disabled
@@ -533,8 +611,8 @@ void Update_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain)
 			/* or the channel is off (volume == 0) */
 			/* or the channel freq is greater than the playback freq */
 			if ( (AUDC[chan + chip_offs] & VOL_ONLY) ||
-				((AUDC[chan + chip_offs] & VOLUME_MASK) == 0) ||
-				(Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))) {
+				((AUDC[chan + chip_offs] & VOLUME_MASK) == 0)
+				/* || (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))*/) {
 				/* indicate the channel is 'on' */
 				Outvol[chan + chip_offs] = 1;
 
@@ -542,8 +620,8 @@ void Update_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain)
 				if ((chan == CHAN3 && !(AUDCTL[chip] & CH1_FILTER)) ||
 					(chan == CHAN4 && !(AUDCTL[chip] & CH2_FILTER)) ||
 					(chan == CHAN1) ||
-					(chan == CHAN2) ||
-					(Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))) {
+					(chan == CHAN2)
+					/* || (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))*/) {
 					/* and set channel freq to max to reduce processing */
 					Div_n_max[chan + chip_offs] = 0x7fffffffL;
 					Div_n_cnt[chan + chip_offs] = 0x7fffffffL;
@@ -574,7 +652,7 @@ void Update_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain)
 /*                                                                           */
 /*****************************************************************************/
 
-void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
+void Pokey_process_8(void * sndbuffer, unsigned sndn)
 {
 	register uint8 *buffer = sndbuffer;
 	register uint16 n = sndn;
@@ -583,14 +661,14 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 	register uint8 *samp_cnt_w_ptr;
 	register uint32 event_min;
 	register uint8 next_event;
-#ifdef CLIP						/* if clipping is selected */
+#ifdef CLIP_SOUND
 	register int16 cur_val;		/* then we have to count as 16-bit signed */
-#ifdef STEREO
+#ifdef STEREO_SOUND
 	register int16 cur_val2;
 #endif
 #else
 	register uint8 cur_val;		/* otherwise we'll simplify as 8-bit unsigned */
-#ifdef STEREO
+#ifdef STEREO_SOUND
 	register int8 cur_val2;
 #endif
 #endif
@@ -615,7 +693,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 	/* output change for increased performance (less over-all math). */
 	/* add the output values of all 4 channels */
 	cur_val = SAMP_MIN;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 	cur_val2 = SAMP_MIN;
 #endif
 
@@ -636,7 +714,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 		if (*out_ptr++)
 			cur_val += *vol_ptr;
 		vol_ptr++;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 		count--;
 		if( count ) {
 			if (*out_ptr++)
@@ -790,7 +868,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 					if (Outvol[next_event & 0xfd]) {
 						/* if on, turn it off */
 						Outvol[next_event & 0xfd] = 0;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 						if( (next_event & 0x04) )
 							cur_val2 -= AUDV[next_event & 0xfd];
 						else
@@ -808,7 +886,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 					if (Outvol[next_event & 0xfd]) {
 						/* if on, turn it off */
 						Outvol[next_event & 0xfd] = 0;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 						if( (next_event & 0x04) )
 							cur_val2 -= AUDV[next_event & 0xfd];
 						else
@@ -822,7 +900,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 			if (toggle) {
 				if (*out_ptr) {
 					/* remove this channel from the signal */
-#ifdef STEREO
+#ifdef STEREO_SOUND
 					if( (next_event & 0x04) )
 						cur_val2 -= AUDV[next_event];
 					else
@@ -837,7 +915,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 					*out_ptr = 1;
 
 					/* and add it to the output signal */
-#ifdef STEREO
+#ifdef STEREO_SOUND
 					if( (next_event & 0x04) )
 						cur_val2 += AUDV[next_event];
 					else
@@ -851,10 +929,10 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 			   which includes an 8 bit fraction for accuracy */
 
 			int iout;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 			int iout2;
 #endif
-#ifndef NOSNDINTER
+#ifdef INTERPOLATE_SOUND
 			if (cur_val != last_val) {
 				if (*Samp_n_cnt < Samp_n_max) {		/* need interpolation */
 					iout = (cur_val * (*Samp_n_cnt) +
@@ -867,7 +945,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 			}
 			else
 				iout = cur_val;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 			if (cur_val2 != last_val2) {
 				if (*Samp_n_cnt < Samp_n_max) {		/* need interpolation */
 					iout2 = (cur_val2 * (*Samp_n_cnt) +
@@ -880,15 +958,15 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 			}
 			else
 				iout2 = cur_val2;
-#endif  /* STEREO */
-#else	/* NOSNDINTER */
+#endif  /* STEREO_SOUND */
+#else   /* INTERPOLATE_SOUND */
 			iout = cur_val;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 			iout2 = cur_val2;
-#endif	/* STEREO */
-#endif	/* NOSNDINTER */
+#endif  /* STEREO_SOUND */
+#endif  /* INTERPOLATE_SOUND */
 
-#ifndef	NO_VOL_ONLY
+#ifdef VOL_ONLY_SOUND
 			if( sampbuf_rptr!=sampbuf_ptr )
 			{ int l;
 				if( sampbuf_cnt[sampbuf_rptr]>0 )
@@ -906,7 +984,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 				}
 			}
 			iout+=sampout;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 			if( sampbuf_rptr2!=sampbuf_ptr2 )
 			{ int l;
 				if( sampbuf_cnt2[sampbuf_rptr2]>0 )
@@ -924,10 +1002,10 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 				}
 			}
 			iout2+=sampout2;
-#endif	/* STEREO */
-#endif	/* NO_VOL_ONLY */
+#endif  /* STEREO_SOUND */
+#endif  /* VOL_ONLY_SOUND */
 
-#ifdef CLIP						/* if clipping is selected */
+#ifdef CLIP_SOUND
 			if (iout > SAMP_MAX) {	/* then check high limit */
 				*buffer++ = (uint8) SAMP_MAX;	/* and limit if greater */
 			}
@@ -937,7 +1015,7 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 			else {				/* otherwise use raw value */
 				*buffer++ = (uint8) iout;
 			}
-#ifdef STEREO
+#ifdef STEREO_SOUND
 			if ((stereo_enabled ? iout2 : iout) > SAMP_MAX) {	/* then check high limit */
 				*buffer++ = (uint8) SAMP_MAX;	/* and limit if greater */
 			}
@@ -947,13 +1025,13 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 			else {				/* otherwise use raw value */
 				*buffer++ = (uint8) (stereo_enabled ? iout2 : iout);
 			}
-#endif /* STEREO */
-#else /* CLIP */
+#endif /* STEREO_SOUND */
+#else /* CLIP_SOUND */
 			*buffer++ = (uint8) iout;	/* clipping not selected, use value */
-#ifdef STEREO
+#ifdef STEREO_SOUND
 			*buffer++ = (uint8) (stereo_enabled ? iout2 : iout);
-#endif /* STEREO */
-#endif /* CLIP */
+#endif /* STEREO_SOUND */
+#endif /* CLIP_SOUND */
 
 #ifdef WORDS_BIGENDIAN
 			*(Samp_n_cnt + 1) += Samp_n_max;
@@ -962,29 +1040,31 @@ void Pokey_process(uint8 * sndbuffer, const uint16 sndn)
 #endif
 			/* and indicate one less byte in the buffer */
 			n--;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 			n--;
 #endif
 		}
 	}
-#ifndef	NO_VOL_ONLY
+#ifdef VOL_ONLY_SOUND
 	if( sampbuf_rptr==sampbuf_ptr )
 		sampbuf_last=cpu_clock;
-#ifdef STEREO
+#ifdef STEREO_SOUND
 	if( sampbuf_rptr2==sampbuf_ptr2 )
 		sampbuf_last2=cpu_clock;
 #endif
-#endif	/* NO_VOL_ONLY */
+#endif  /* VOL_ONLY_SOUND */
 
 	if( IsSoundFileOpen())
 		WriteToSoundFile(sndbuffer, sndn);
 }
 
 #ifdef SERIO_SOUND
-void Update_serio_sound( int out, UBYTE data )
+static void Update_serio_sound_rf( int out, UBYTE data )
 {
-#ifndef	NO_VOL_ONLY
+#ifdef VOL_ONLY_SOUND
    int bits,pv,future;
+        if (!serio_sound_enabled) return;
+
 	pv=0;
 	future=0;
 	bits= (data<<1) | 0x200;
@@ -1011,17 +1091,39 @@ void Update_serio_sound( int out, UBYTE data )
 		bits>>=1;
 	}
 	sampbuf_lastval-=pv;
-#endif	/* NO_VOL_ONLY */
+#endif  /* VOL_ONLY_SOUND */
 }
 #endif /* SERIO_SOUND */
 
-#ifndef NO_CONSOL_SOUND
-void Update_consol_sound( int set )
+void Pokey_process_16(void * sndbuffer, unsigned sndn)
+{
+  uint16 *buffer = sndbuffer;
+  int i;
+
+  Pokey_process_8(buffer, sndn);
+
+  for (i = sndn - 1; i >= 0; i--)
+  {
+    int smp = ((int)(((uint8 *)buffer)[i]) - 0x80) * 0x100;
+
+    if (smp > SHRT_MAX)
+      smp = SHRT_MAX;
+    if (smp < SHRT_MIN)
+      smp = SHRT_MIN;
+
+    buffer[i] = smp;
+  }
+}
+
+#ifdef CONSOLE_SOUND
+static void Update_consol_sound_rf(int set)
 { 
-#ifndef	NO_VOL_ONLY
+#ifdef VOL_ONLY_SOUND
   static int prev_atari_speaker=0;
   static unsigned int prev_cpu_clock=0;
   int d;
+        if (!console_sound_enabled) return;
+        
 	if( !set && samp_consol_val==0 )	return;
 	sampbuf_lastval-=samp_consol_val;
 	if( prev_atari_speaker!=atari_speaker )
@@ -1053,21 +1155,52 @@ void Update_consol_sound( int set )
 		if( sampbuf_rptr>=SAMPBUF_MAX )
 			sampbuf_rptr=0;
 	}
-#endif	/* NO_VOL_ONLY */
+#endif  /* VOL_ONLY_SOUND */
 }
-#endif /* NO_CONSOL_SOUND */
+#endif /* CONSOLE_SOUND */
 
-#ifndef	NO_VOL_ONLY
-void Update_vol_only_sound( void )
+#ifdef VOL_ONLY_SOUND
+static void Update_vol_only_sound_rf(void)
 {
-#ifndef NO_CONSOL_SOUND
+#ifdef CONSOLE_SOUND
 	Update_consol_sound(0);	/* mmm */
-#endif /* NO_CONSOL_SOUND */
+#endif /* CONSOLE_SOUND */
 }
-#endif	/* NO_VOL_ONLY */
+#endif  /* VOL_ONLY_SOUND */
 
 /*
 $Log: pokeysnd.c,v $
+Revision 1.18  2003/12/12 00:23:59  markgrebe
+Added enable for console and sio sound
+
+Revision 1.17  2003/02/24 09:33:09  joy
+header cleanup
+
+Revision 1.16  2003/02/19 14:07:47  joy
+configure stuff cleanup
+
+Revision 1.15  2003/02/10 14:59:17  joy
+fixes fixes
+
+Revision 1.14  2003/02/10 13:29:16  joy
+extern removed
+
+Revision 1.13  2003/02/09 21:20:43  joy
+updated for global enable_new_pokey
+
+Revision 1.12  2003/02/09 13:17:29  joy
+switch Pokey cores on-the-fly
+
+Revision 1.11  2003/02/08 23:52:17  joy
+little cleanup
+
+Revision 1.10  2002/12/08 20:32:02  knik
+added 16bit and quality sound setting support
+added multiple sound engines support
+
+Revision 1.9  2002/08/26 05:44:22  pfusik
+Adam Bienias's fix for better sound quality
+
 Revision 1.8  2001/07/22 08:24:47  knik
 PURE -> PURETONE to avoid windows headers interference
 
