@@ -25,9 +25,14 @@ extern int disable_basic;
 extern int enable_sio_patch;
 extern int enable_h_patch;
 extern int enable_p_patch;
+extern int enable_r_patch;
+extern int enable_new_pokey;
+extern int stereo_enabled;
 extern int disk_directories;
+extern int serio_sound_enabled;
+extern int console_sound_enabled;
 
-int RtConfigLoad(char *rtconfig_filename);
+int RtConfigLoad(const char *alternate_config_filename);
 void RtConfigSave(void);
 void RtConfigUpdate(void);
 
