@@ -1,9 +1,38 @@
-/* $Id: sio.c,v 1.14 2001/10/03 16:46:09 fox Exp $ */
+/*
+ * sio.c - Serial I/O emulation
+ *
+ * Copyright (C) 1995-1998 David Firth
+ * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <unistd.h>
+#ifdef DEBUG
+#include <time.h>
+#include <sys/time.h>
+#endif
 
+#include "antic.h"  /* ypos */
 #include "atari.h"
 #include "config.h"
 #include "cpu.h"
@@ -13,9 +42,13 @@
 #include "pokeysnd.h"
 #include "platform.h"
 #include "log.h"
-#include "diskled.h"
 #include "binload.h"
 #include "cassette.h"
+#include "statesav.h"
+
+#ifdef SHOW_DISK_LED
+#include "diskled.h"
+#endif
 
 /* If ATR image is in double density (256 bytes per sector),
    then the boot sectors (sectors 1-3) can be:
@@ -42,6 +75,10 @@ static int sectorcount[MAX_DRIVES];
 static int sectorsize[MAX_DRIVES];
 static int format_sectorcount[MAX_DRIVES];
 static int format_sectorsize[MAX_DRIVES];
+static int io_success[MAX_DRIVES];
+#ifdef DEBUG
+static struct timeval tv;
+#endif
 
 UnitStatus drive_status[MAX_DRIVES];
 char sio_filename[MAX_DRIVES][FILENAME_MAX];
@@ -58,6 +95,8 @@ int DataIndex = 0;
 int TransferStatus = 0;
 int ExpectedBytes = 0;
 
+int ignore_header_writeprotect = 0;
+
 extern FILE *opendcm( int diskno, const char *infilename, char *outfilename );
 #ifdef HAVE_LIBZ
 extern FILE *openzlib(int diskno, const char *infilename, char *outfilename );
@@ -158,7 +197,7 @@ int SIO_Mount(int diskno, const char *filename, int b_open_readonly)
 		if ((header.magic1 == MAGIC1) && (header.magic2 == MAGIC2)) {
 			format[diskno - 1] = ATR;
 
-			if (header.writeprotect)
+			if (header.writeprotect && !ignore_header_writeprotect)
 				drive_status[diskno - 1] = ReadOnly;
 
 			sectorsize[diskno - 1] = header.secsizehi << 8 |
@@ -305,12 +344,16 @@ static int ReadSector(int unit, int sector, UBYTE * buffer)
 	if (start_binloading)
 		return BIN_loade_start(buffer);
 
+	io_success[unit]=-1;
 	if (drive_status[unit] != Off) {
 		if (disk[unit]) {
 			if (sector > 0 && sector <= sectorcount[unit]) {
+#ifdef SHOW_DISK_LED
 				LED_SetRead(unit, 1);
+#endif
 				size = SeekSector(unit, sector);
 				fread(buffer, 1, size, disk[unit]);
+				io_success[unit]=0;
 				return 'C';
 			}
 			else
@@ -327,13 +370,17 @@ static int WriteSector(int unit, int sector, UBYTE * buffer)
 {
 	int size;
 
+	io_success[unit]=-1;
 	if (drive_status[unit] != Off) {
 		if (disk[unit]) {
 			if (drive_status[unit] == ReadWrite) {
+#ifdef SHOW_DISK_LED
 				LED_SetWrite(unit, 1);
+#endif
 				if (sector > 0 && sector <= sectorcount[unit]) {
 					size = SeekSector(unit, sector);
 					fwrite(buffer, 1, size, disk[unit]);
+					io_success[unit]=0;
 					return 'C';
 				}
 				else
@@ -351,6 +398,7 @@ static int WriteSector(int unit, int sector, UBYTE * buffer)
 
 static int FormatDisk(int unit, UBYTE *buffer, int sectsize, int sectcount)
 {
+	io_success[unit]=-1;
 	if (drive_status[unit] != Off) {
 		if (disk[unit]) {
 			if (drive_status[unit] == ReadWrite) {
@@ -411,6 +459,7 @@ static int FormatDisk(int unit, UBYTE *buffer, int sectsize, int sectcount)
 					boot_sectors_type[unit] = save_boot_sectors_type;
 				/* Return information for Atari (buffer filled with ff's - no bad sectors) */
 				memset(buffer, 0xff, sectsize);
+				io_success[unit]=0;
 				return 'C';
 			}
 			else
@@ -433,6 +482,12 @@ static int WriteStatusBlock(int unit, UBYTE * buffer)
 {
 	int size;
 
+#ifdef DEBUG
+	Aprint("Write Status-Block: %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x",
+		buffer[0], buffer[1], buffer[2], buffer[3],
+		buffer[4], buffer[5], buffer[6], buffer[7],
+		buffer[8], buffer[9], buffer[10], buffer[11]);
+#endif
 	if (drive_status[unit] != Off) {
 		/*
 		 * We only care about the density and the sector count
@@ -440,7 +495,7 @@ static int WriteStatusBlock(int unit, UBYTE * buffer)
 		 * be non-sense
 		 */
 		/* I'm not sure about this density settings, my XF551
-		 * honnors only the sector size and ignore the density
+		 * honors only the sector size and ignores the density
 		 */
 		size = buffer[6] * 256 + buffer[7];
 		if (size == 128 || size == 256)
@@ -515,14 +570,14 @@ static int ReadStatusBlock(int unit, UBYTE * buffer)
 
    Bit 0 = 1 indicates an invalid command frame was received
    Bit 1 = 1 indicates an invalid data frame was received
-   Bit 2 = 1 indicates that a PUT operation was unsuccessful
-   Bit 3 = 1 indicates that the diskete is write protected
+   Bit 2 = 1 indicates that last read/write operation was unsuccessful
+   Bit 3 = 1 indicates that the diskette is write protected
    Bit 4 = 1 indicates active/standby
 
    plus
 
    Bit 5 = 1 indicates double density
-   Bit 7 = 1 indicates duel density disk (1050 format)
+   Bit 7 = 1 indicates dual density disk (1050 format)
  */
 static int DriveStatus(int unit, UBYTE * buffer)
 {
@@ -537,6 +592,8 @@ static int DriveStatus(int unit, UBYTE * buffer)
 	if (drive_status[unit] != Off) {
 		buffer[0] = 16;			/* drive active */
 		buffer[1] = 255;		/* WD 177x OK */
+		if (io_success[unit]!=0)
+			buffer[0] |= 4;         /* failed RW-operation */
 		if (drive_status[unit] == ReadOnly)
 			buffer[0] |= 8;		/* write protection */
 		if (sectorsize[unit] == 256)
@@ -553,6 +610,14 @@ static int DriveStatus(int unit, UBYTE * buffer)
 		return 0;
 }
 
+#ifndef NO_SECTOR_DELAY
+/* A hack for the "Overmind" demo.  This demo verifies if sectors aren't read
+   faster than with a typical disk drive.  We introduce a delay
+   of SECTOR_DELAY scanlines between successive reads of sector 1. */
+#define SECTOR_DELAY 3200
+static int delay_counter = 0;
+static int last_ypos = 0;
+#endif
 
 void SIO(void)
 {
@@ -563,11 +628,16 @@ void SIO(void)
 	int length = dGetWordAligned(0x308);
 	int realsize = 0;
 	int cmd = dGetByte(0x302);
-#ifndef NO_SECTOR_DELAY
-	static int delay_counter = 1;	/* no delay on first read */
-#endif
 
-	if (dGetByte(0x300) == 0x31 && unit < MAX_DRIVES)	/* UBYTE range ! */
+	if (dGetByte(0x300) == 0x31 && unit < MAX_DRIVES) {	/* UBYTE range ! */
+#ifdef DEBUG
+		gettimeofday(&tv,NULL);
+		Aprint("%04lu.%03lu SIO disk command is %02x %02x %02x %02x %02x   %02x %02x %02x %02x %02x %02x",
+			tv.tv_sec%10000,tv.tv_usec/1000,
+			cmd,dGetByte(0x303),dGetByte(0x304),dGetByte(0x305),dGetByte(0x306),
+			dGetByte(0x308),dGetByte(0x309),dGetByte(0x30A),dGetByte(0x30B),
+			dGetByte(0x30C),dGetByte(0x30D));
+#endif
 		switch (cmd) {
 		case 0x4e:				/* Read Status Block */
 			if (12 == length) {
@@ -588,6 +658,8 @@ void SIO(void)
 			break;
 		case 0x50:				/* Write */
 		case 0x57:
+		case 0xD0:				/* xf551 hispeed */
+		case 0xD7:
 			SizeOfSector(unit, sector, &realsize, NULL);
 			if (realsize == length) {
 				CopyFromMem(data, DataBuffer, realsize);
@@ -597,16 +669,21 @@ void SIO(void)
 				result = 'E';
 			break;
 		case 0x52:				/* Read */
+		case 0xD2:				/* xf551 hispeed */
 #ifndef NO_SECTOR_DELAY
-			if (sector == 2) {
-				if ((xpos = xpos_limit) == LINE_C)
-					delay_counter--;
-				if (delay_counter) {
-					regPC = 0xe459;	/* stay in SIO */
+			if (sector == 1) {
+				if (delay_counter > 0) {
+					if (last_ypos != ypos) {
+						last_ypos = ypos;
+						delay_counter--;
+					}
+					regPC = 0xe459;	/* stay at SIO patch */
 					return;
 				}
-				else
-					delay_counter = SECTOR_DELAY;
+				delay_counter = SECTOR_DELAY;
+			}
+			else {
+				delay_counter = 0;
 			}
 #endif
 			SizeOfSector(unit, sector, &realsize, NULL);
@@ -628,32 +705,75 @@ void SIO(void)
 			break;
 		/*case 0x66:*/			/* US Doubler Format - I think! */
 		case 0x21:				/* Format Disk */
+		case 0xA1:				/* xf551 hispeed */
 			realsize = format_sectorsize[unit];
 			if (realsize == length) {
 				result = FormatDisk(unit, DataBuffer, realsize, format_sectorcount[unit]);
 				if (result == 'C')
 					CopyToMem(DataBuffer, data, realsize);
 			}
-			else
+			else {
+				//there are programs which send the format-command but dont wait for the result (eg xf-tools)
+				FormatDisk(unit, DataBuffer, realsize, format_sectorcount[unit]);
 				result = 'E';
+			}
 			break;
 		case 0x22:				/* Enhanced Density Format */
+		case 0xA2:				/* xf551 hispeed */
 			realsize = 128;
 			if (realsize == length) {
 				result = FormatDisk(unit, DataBuffer, 128, 1040);
 				if (result == 'C')
 					CopyToMem(DataBuffer, data, realsize);
 			}
-			else
+			else {
+				FormatDisk(unit, DataBuffer, 128, 1040);
 				result = 'E';
+			}
 			break;
 		default:
 			result = 'N';
 		}
+	}
+	//cassette i/o
 	else if (dGetByte(0x300) == 0x60) {
-		result = CASSETTE_Sio();
-		if (result == 'C')
-			CopyToMem(cassette_buffer, data, length);
+		int storagelength=0;
+		UBYTE gaps=dGetByte(0x30b);
+		switch(cmd){
+		case 0x52:	//read
+			//set expected Gap
+			CASSETTE_AddGap(gaps==0?2000:160);
+			//get record from storage medium
+			storagelength = CASSETTE_Read();
+			if(storagelength-1!=length)	//includes -1 as error
+				result = 'E';
+			else
+				result = 'C';
+			//check checksum
+			if(cassette_buffer[length]!=SIO_ChkSum(cassette_buffer,length)){
+				result = 'E';
+			};
+			//if all went ok, copy to Atari
+			if (result == 'C')
+				CopyToMem(cassette_buffer, data, length);
+			break;
+		case 0x57:	//write
+			//put record into buffer
+			CopyFromMem(data, cassette_buffer, length);
+			//eval checksum over buffer data
+			cassette_buffer[length]=SIO_ChkSum(cassette_buffer,length);
+			//add pregap length
+			CASSETTE_AddGap(gaps==0?3000:260);
+			//write full record to storage medium
+			storagelength = CASSETTE_Write(length+1);
+			if(storagelength-1!=length)	//includes -1 as error
+				result = 'E';
+			else
+				result = 'C';
+			break;
+		default:
+			result = 'N';
+		};
 	}
 
 	switch (result) {
@@ -704,6 +824,9 @@ static UBYTE Command_Frame(void)
 
 	sector = CommandFrame[2] | (((UWORD) (CommandFrame[3])) << 8);
 	unit = CommandFrame[0] - '1';
+#ifdef DEBUG
+	gettimeofday(&tv,NULL);
+#endif
 	if (unit < 0 || unit >= MAX_DRIVES) {
 		/* Unknown device */
 		Aprint("Unknown command frame: %02x %02x %02x %02x %02x",
@@ -715,6 +838,12 @@ static UBYTE Command_Frame(void)
 	else
 		switch (CommandFrame[1]) {
 		case 0x4e:				/* Read Status */
+#ifdef DEBUG
+			Aprint("%04lu.%03lu Read-status frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
+#endif
 			DataBuffer[0] = ReadStatusBlock(unit, DataBuffer + 1);
 			DataBuffer[13] = SIO_ChkSum(DataBuffer + 1, 12);
 			DataIndex = 0;
@@ -722,34 +851,71 @@ static UBYTE Command_Frame(void)
 			TransferStatus = SIO_ReadFrame;
 			DELAYED_SERIN_IRQ = SERIN_INTERVAL;
 			return 'A';
-		case 0x4f:
+		case 0x4f:				/* Write status */
+#ifdef DEBUG
+			Aprint("%04lu.%03lu Write-status frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
+#endif
 			ExpectedBytes = 13;
 			DataIndex = 0;
 			TransferStatus = SIO_WriteFrame;
 			return 'A';
 		case 0x50:				/* Write */
 		case 0x57:
+		case 0xD0:				/* xf551 hispeed */
+		case 0xD7:
+#ifdef DEBUG
+			Aprint("%04lu.%03lu Write-sector frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
+#endif
 			SizeOfSector((UBYTE)unit, sector, &realsize, NULL);
 			ExpectedBytes = realsize + 1;
 			DataIndex = 0;
 			TransferStatus = SIO_WriteFrame;
+#ifdef SHOW_DISK_LED
 			LED_SetWrite(unit, 10);
+#endif
 			return 'A';
 		case 0x52:				/* Read */
+		case 0xD2:				/* xf551 hispeed */
+#ifdef DEBUG
+			Aprint("%04lu.%03lu Read-sector frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
+#endif
 			SizeOfSector((UBYTE)unit, sector, &realsize, NULL);
 			DataBuffer[0] = ReadSector(unit, sector, DataBuffer + 1);
 			DataBuffer[1 + realsize] = SIO_ChkSum(DataBuffer + 1, (UWORD)realsize);
 			DataIndex = 0;
 			ExpectedBytes = 2 + realsize;
 			TransferStatus = SIO_ReadFrame;
-			DELAYED_SERIN_IRQ = SERIN_INTERVAL;
+			//wait longer before confirmation because bytes could be lost before the buffer was set (see $E9FB & $EA37 in XL-OS)
+			DELAYED_SERIN_IRQ = SERIN_INTERVAL << 2;
 #ifndef NO_SECTOR_DELAY
-			if (sector == 2)
-				DELAYED_SERIN_IRQ += SECTOR_DELAY;
+			if (sector == 1) {
+				DELAYED_SERIN_IRQ += delay_counter;
+				delay_counter = SECTOR_DELAY;
+			}
+			else {
+				delay_counter = 0;
+			}
 #endif
+#ifdef SHOW_DISK_LED
 			LED_SetRead(unit, 10);
+#endif
 			return 'A';
 		case 0x53:				/* Status */
+#ifdef DEBUG
+			Aprint("%04lu.%03lu Status frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
+#endif
 			DataBuffer[0] = DriveStatus(unit, DataBuffer + 1);
 			DataBuffer[1 + 4] = SIO_ChkSum(DataBuffer + 1, 4);
 			DataIndex = 0;
@@ -759,6 +925,13 @@ static UBYTE Command_Frame(void)
 			return 'A';
 		/*case 0x66:*/			/* US Doubler Format - I think! */
 		case 0x21:				/* Format Disk */
+		case 0xa1:				/* xf551 hispeed */
+#ifdef DEBUG
+			Aprint("%04lu.%03lu Format-disk frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
+#endif
 			realsize = format_sectorsize[unit];
 			DataBuffer[0] = FormatDisk(unit, DataBuffer + 1, realsize, format_sectorcount[unit]);
 			DataBuffer[1 + realsize] = SIO_ChkSum(DataBuffer + 1, realsize);
@@ -767,7 +940,14 @@ static UBYTE Command_Frame(void)
 			TransferStatus = SIO_FormatFrame;
 			DELAYED_SERIN_IRQ = SERIN_INTERVAL;
 			return 'A';
-		case 0x22:				/* Duel Density Format */
+		case 0x22:				/* Dual Density Format */
+		case 0xa2:				/* xf551 hispeed */
+#ifdef DEBUG
+			Aprint("%04lu.%03lu Format-Medium frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
+#endif
 			DataBuffer[0] = FormatDisk(unit, DataBuffer + 1, 128, 1040);
 			DataBuffer[1 + 128] = SIO_ChkSum(DataBuffer + 1, 128);
 			DataIndex = 0;
@@ -778,9 +958,10 @@ static UBYTE Command_Frame(void)
 		default:
 			/* Unknown command for a disk drive */
 #ifdef DEBUG
-			Aprint("Command frame: %02x %02x %02x %02x %02x",
-				   CommandFrame[0], CommandFrame[1], CommandFrame[2],
-				   CommandFrame[3], CommandFrame[4]);
+			Aprint("%04lu.%03lu Command frame: %02x %02x %02x %02x %02x",
+				tv.tv_sec%10000, tv.tv_usec/1000,
+				CommandFrame[0], CommandFrame[1], CommandFrame[2],
+				CommandFrame[3], CommandFrame[4]);
 #endif
 			TransferStatus = SIO_NoFrame;
 			return 'E';
@@ -793,7 +974,7 @@ void SwitchCommandFrame(int onoff)
 
 	if (onoff) {				/* Enabled */
 		if (TransferStatus != SIO_NoFrame)
-			Aprint("Unexpected command frame %x.", TransferStatus);
+			Aprint("Unexpected command frame at state %x.", TransferStatus);
 		CommandIndex = 0;
 		DataIndex = 0;
 		ExpectedBytes = 5;
@@ -828,6 +1009,8 @@ static UBYTE WriteSectorBack(void)
 			break;
 		case 0x50:				/* Write */
 		case 0x57:
+		case 0xD0:				/* xf551 hispeed */
+		case 0xD7:
 			result = WriteSector(unit, sector, DataBuffer);
 			break;
 		default:
@@ -915,7 +1098,8 @@ int SIO_GetByte(void)
 				TransferStatus = SIO_NoFrame;
 			}
 			else {
-				DELAYED_SERIN_IRQ = SERIN_INTERVAL;
+				//set delay using the expected transfer speed
+				DELAYED_SERIN_IRQ = (DataIndex==1) ? SERIN_INTERVAL : ((SERIN_INTERVAL*AUDF[CHAN3]-1)/0x28+1);
 			}
 		}
 		else {
@@ -987,8 +1171,88 @@ int Rotate_Disks( void )
 	return bSuccess;
 }
 
+void SIOStateSave( void ) 
+{
+    int i;
+    UWORD namelen;    
+    char *filename;
+    char dirname[FILENAME_MAX];
+
+    getcwd(dirname, FILENAME_MAX);
+
+    for (i=0;i<8;i++) {
+        if (strncmp(sio_filename[i], dirname, strlen(dirname)) == 0)
+            filename = &sio_filename[i][strlen(dirname)+1];
+        else
+            filename = sio_filename[i];
+
+        namelen = strlen(filename);
+        SaveINT((int *)&drive_status[i], 1);
+        SaveUWORD(&namelen, 1);
+        SaveUBYTE(filename, namelen);
+        }
+}
+
+void SIOStateRead( void )
+{
+    int i;
+    UWORD namelen;
+    char filename[FILENAME_MAX];
+    FILE *fp;
+    int saved_drive_status;
+
+    for (i=0;i<8;i++) {
+        ReadINT((int *)&saved_drive_status, 1);
+        ReadUWORD(&namelen,1);
+        if (namelen != 0) {
+            filename[0] = 0;
+            ReadUBYTE(filename, namelen);
+            filename[namelen] = 0;
+            /* Check to make sure the filename is valid */
+            fp = fopen(filename, "rb");
+            if (fp == NULL)
+		continue;
+            else
+                fclose(fp);                
+            drive_status[i] = saved_drive_status;
+            /* If the disk drive wasn't empty or off when saved,
+               mount the disk */
+            if ((drive_status[i] != Off) &&
+                (drive_status[i] != NoDisk)) {
+                    /* unmount */
+                    SIO_Dismount(i + 1);
+                    /* try to mount with saved R/W status  */
+                    if (drive_status[i] == ReadOnly)
+                        SIO_Mount(i + 1, filename, 1);
+                    else
+                        SIO_Mount(i + 1, filename, 0);
+                }
+            }
+        else
+            drive_status[i] = saved_drive_status;
+        }
+}
+
 /*
 $Log: sio.c,v $
+Revision 1.20  2003/11/22 23:26:19  joy
+cassette support improved
+
+Revision 1.19  2003/10/26 18:49:40  joy
+new state file format for bankswitching
+
+Revision 1.18  2003/10/24 14:13:14  pfusik
+"Overmind" runs with NEW_CYCLE_EXACT and SIO patch enabled
+
+Revision 1.17  2003/08/05 11:29:04  joy
+XF551 highspeed transfer
+
+Revision 1.16  2003/03/03 09:57:33  joy
+Ed improved autoconf again plus fixed some little things
+
+Revision 1.15  2003/02/24 09:33:10  joy
+header cleanup
+
 Revision 1.14  2001/10/03 16:46:09  fox
 SET_LED and Atari_Set_LED are no longer used
 
