@@ -1,4 +1,27 @@
-/* $Id: ui.c,v 1.27 2001/11/18 19:35:59 fox Exp $ */
+/*
+ * ui.c - main user interface
+ *
+ * Copyright (C) 1995-1998 David Firth
+ * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
@@ -24,23 +47,24 @@
 #include "binload.h"
 #include "sndsave.h"
 #include "cartridge.h"
+#include "cassette.h"
 #include "rtime.h"
 #include "input.h"
+#include "pokeysnd.h"
+#include "rt-config.h"	/* extern for enable_new_pokey and stereo_enabled */
 
 tUIDriver* ui_driver = &basic_ui_driver;
 
 int ui_is_active = FALSE;
 int alt_function = -1;		/* alt function init */
 int current_disk_directory = 0;
-
-#ifdef STEREO
-extern int stereo_enabled;
-#endif
+int hold_start_on_reboot = 0;
 
 static char curr_disk_dir[FILENAME_MAX] = "";
 static char curr_cart_dir[FILENAME_MAX] = "";
 static char curr_exe_dir[FILENAME_MAX] = "";
 static char curr_state_dir[FILENAME_MAX] = "";
+static char curr_tape_dir[FILENAME_MAX] = "";
 
 #ifdef CRASH_MENU
 int crash_code=-1;
@@ -53,9 +77,9 @@ int CrashMenu();
 void DiskManagement();
 void CartManagement();
 int RunExe();
+int LoadTape();
 void SelectSystem();
-void SetSoundType();
-void SoundRecording();
+int SoundSettings();
 void SelectArtifacting();
 void AtariSettings();
 int SaveState();
@@ -73,28 +97,38 @@ void SelectSystem()
 
 	static tMenuItem menu_array[] =
 	{
-		{ "SYAS", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/A (48 KB)",              NULL, 0 },
-		{ "SYAL", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/A (52 KB)",              NULL, 1 },
-		{ "SYBS", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/B (48 KB)",              NULL, 2 },
-		{ "SYBL", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/B (52 KB)",              NULL, 3 },
-		{ "SYXL", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 800XL (64 KB)",             NULL, 4 },
-		{ "SYXE", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 130XE (128 KB)",            NULL, 5 },
-		{ "SYRM", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 320XE (320 KB RAMBO)",      NULL, 6 },
-		{ "SYCS", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 320XE (320 KB COMPY SHOP)", NULL, 7 },
-		{ "SY52", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 5200 (16 KB)",              NULL, 8 },
+		{ "SYAF", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/A (16 KB)",              NULL, 0 },
+		{ "SYAS", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/A (48 KB)",              NULL, 1 },
+		{ "SYAL", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/A (52 KB)",              NULL, 2 },
+		{ "SYBF", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/B (16 KB)",              NULL, 3 },
+		{ "SYBS", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/B (48 KB)",              NULL, 4 },
+		{ "SYBL", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari OS/B (52 KB)",              NULL, 5 },
+		{ "SYXS", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 600XL (16 KB)",             NULL, 6 },
+		{ "SYXL", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 800XL (64 KB)",             NULL, 7 },
+		{ "SYXE", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 130XE (128 KB)",            NULL, 8 },
+		{ "SYRM", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 320XE (320 KB RAMBO)",      NULL, 9 },
+		{ "SYCS", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 320XE (320 KB COMPY SHOP)", NULL, 10 },
+		{ "SY05", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 576XE (576 KB)",            NULL, 11 },
+		{ "SY1M", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 1088XE (1088 KB)",          NULL, 12 },
+		{ "SY52", ITEM_ENABLED|ITEM_ACTION, NULL, "Atari 5200 (16 KB)",              NULL, 13 },
 		MENU_END
 	};
 
 	static tSysConfig machine[] =
 	{
+		{ MACHINE_OSA,  16 },
 		{ MACHINE_OSA,  48 },
 		{ MACHINE_OSA,  52 },
+		{ MACHINE_OSB,  16 },
 		{ MACHINE_OSB,  48 },
 		{ MACHINE_OSB,  52 },
+		{ MACHINE_XLXE, 16 },
 		{ MACHINE_XLXE, 64 },
 		{ MACHINE_XLXE, 128 },
 		{ MACHINE_XLXE, RAM_320_RAMBO },
 		{ MACHINE_XLXE, RAM_320_COMPY_SHOP },
+		{ MACHINE_XLXE, 576 },
+		{ MACHINE_XLXE, 1088 },
 		{ MACHINE_5200, 16 }
 	};
 
@@ -132,6 +166,9 @@ void DiskManagement()
 		{ "DKS6", ITEM_ENABLED|ITEM_FILESEL|ITEM_MULTI, drive_array[5], sio_filename[5], NULL, 5 },
 		{ "DKS7", ITEM_ENABLED|ITEM_FILESEL|ITEM_MULTI, drive_array[6], sio_filename[6], NULL, 6 },
 		{ "DKS8", ITEM_ENABLED|ITEM_FILESEL|ITEM_MULTI, drive_array[7], sio_filename[7], NULL, 7 },
+        { "SSET", ITEM_ENABLED|ITEM_FILESEL|ITEM_MULTI, NULL, "Save Disk Set", NULL, 8 },
+        { "LSET", ITEM_ENABLED|ITEM_FILESEL|ITEM_MULTI, NULL, "Load Disk Set", NULL, 9 },
+		{ "RDSK", ITEM_ENABLED|ITEM_ACTION, NULL, "Rotate Disks", NULL, 10 },
 		MENU_END
 	};
 
@@ -147,7 +184,10 @@ void DiskManagement()
 
 	while (!done) {
 		char filename[FILENAME_MAX+1];
+		char diskfilename[FILENAME_MAX+1];
+        char setname[FILENAME_MAX+1];
 		int seltype;
+        FILE *setFile;
 
 		for(i = 0; i < 8; i++)
 		{
@@ -157,7 +197,10 @@ void DiskManagement()
 		dsknum = ui_driver->fSelect("Disk Management", FALSE, dsknum, menu_array, &seltype);
 
 		if (dsknum > -1) {
-			if (seltype == USER_SELECT) {	/* User pressed "Enter" to select a disk image */
+			if (dsknum == 10) {
+				Rotate_Disks();
+			}
+			else if (seltype == USER_SELECT) {	/* User pressed "Enter" to select a disk image */
 				char *pathname;
 
 /*              pathname=atari_disk_dirs[current_disk_directory]; */
@@ -165,33 +208,94 @@ void DiskManagement()
 				if (curr_disk_dir[0] == '\0')
 					strcpy(curr_disk_dir, atari_disk_dirs[current_disk_directory]);
 
-				while (ui_driver->fGetLoadFilename(curr_disk_dir, filename)) {
-					DIR *subdir;
+                if (dsknum == 8) {  /* Save a disk set */
+                    /* Get the filename */
+                    ui_driver->fGetSaveFilename(filename);
 
-					subdir = opendir(filename);
-					if (!subdir) {	/* A file was selected */
-						SIO_Dismount(dsknum + 1);
-						SIO_Mount(dsknum + 1, filename, rwflags[dsknum]);
-						break;
-					}
-					else {		/* A directory was selected */
-						closedir(subdir);
-						pathname = filename;
-					}
-				}
+                    /* Put it in the current disks directory */
+                    strcpy(setname, curr_disk_dir);
+                    if (*setname) {
+                        char last = setname[strlen(setname)-1];
+                        if (last != '/' && last != '\\')
+                #ifdef BACK_SLASH
+                            strcat(setname, "\\");
+                #else
+                            strcat(setname, "/");
+                #endif
+                    }
+                    strcat(setname, filename);
+
+                    /* Write the current disk file names out to it */
+                    setFile = fopen(setname,"w");
+                    if (setFile) {
+                        for (i=0;i<8;i++) 
+                            fprintf(setFile,"%s\n",sio_filename[i]);
+                        }
+                        fclose(setFile);
+                    }
+                else {
+				    while (ui_driver->fGetLoadFilename(curr_disk_dir, filename)) {
+                        DIR *subdir;
+
+                        subdir = opendir(filename);
+                        if (!subdir) {	/* A file was selected */
+                            if (dsknum < 8) { /* Normal disk mount */
+                                SIO_Dismount(dsknum + 1);
+                                /* try to mount read/write */
+                                SIO_Mount(dsknum + 1, filename, FALSE);
+                                /* update rwflags with the real mount status */
+                                rwflags[dsknum] = (drive_status[dsknum] == ReadOnly ? TRUE : FALSE);
+                                }
+                            else if (dsknum == 9) { /* Load a disk set */
+                                setFile = fopen(filename,"r");
+                                if (setFile) {
+                                    for (i=0;i<8;i++) {
+                                        /* Get the disk filename from the set file */
+                                        fgets(diskfilename,FILENAME_MAX,setFile);
+                                        /* Remove the trailing newline */
+                                        if (strlen(diskfilename) != 0) 
+                                            diskfilename[strlen(diskfilename)-1] = 0;
+                                        /* If the disk drive wasn't empty or off when saved,
+                                           mount the disk */
+                                        if ((strcmp(diskfilename,"Empty") != 0) &&
+                                            (strcmp(diskfilename,"Off") != 0)) {
+                                            SIO_Dismount(i+1);
+                                            /* Mount the disk read/write */
+                                            SIO_Mount(i+1, diskfilename, FALSE);
+                                            /* update rwflags with the real mount status */
+                                            rwflags[i] = (drive_status[i] == ReadOnly ? TRUE : FALSE);
+                                            }
+                                        }
+                                    fclose(setFile);
+                                    }
+                                }
+                             break;
+                            }
+                        else {		/* A directory was selected */
+                            closedir(subdir);
+                            pathname = filename;
+                        }
+                    }
+                }
 			}
-			else if (seltype == USER_TOGGLE) {	/* User pressed "SpaceBar" to change read/write flag of this drive */
-				rwflags[dsknum] = !rwflags[dsknum];
-				/* now the drive should probably be remounted
-				   and the rwflag should be read from the drive_status again */
-				/* TODO remount drive */
-				menu_array[dsknum].prefix[1] = rwflags[dsknum] ? 'R' : 'W';
+			else if (seltype == USER_TOGGLE) {
+                if (dsknum < 8) {
+                    /* User pressed "SpaceBar" to change R/W status of this drive */
+                    /* unmount */
+                    SIO_Dismount(dsknum + 1);
+                    /* try to remount with changed R/W status (!rwflags) */
+                    SIO_Mount(dsknum + 1, filename, !rwflags[dsknum]);
+                    /* update rwflags with the real mount status */
+                    rwflags[dsknum] = (drive_status[dsknum] == ReadOnly ? TRUE : FALSE);
+                    }
 			}
 			else {
-				if (strcmp(sio_filename[dsknum], "Empty") == 0)
-					SIO_DisableDrive(dsknum + 1);
-				else
-					SIO_Dismount(dsknum + 1);
+                if (dsknum < 8) {
+                    if (strcmp(sio_filename[dsknum], "Empty") == 0)
+                        SIO_DisableDrive(dsknum + 1);
+                    else
+                        SIO_Dismount(dsknum + 1);
+                }
 			}
 		}
 		else
@@ -211,7 +315,7 @@ int SelectCartType(UBYTE* screen, int k)
 		{ "CRT5", ITEM_ACTION, NULL, "DB 32 KB cartridge",              NULL, 5 },
 		{ "CRT6", ITEM_ACTION, NULL, "Two chip 16 KB 5200 cartridge",   NULL, 6 },
 		{ "CRT7", ITEM_ACTION, NULL, "Bounty Bob 40 KB 5200 cartridge", NULL, 7 },
-		{ "CRT8", ITEM_ACTION, NULL, "8*8 KB D50x 64 KB cartridge",     NULL, 8 },
+		{ "CRT8", ITEM_ACTION, NULL, "64 KB Williams cartridge",     	NULL, 8 },
 		{ "CRT9", ITEM_ACTION, NULL, "Express 64 KB cartridge",         NULL, 9 },
 		{ "CRTA", ITEM_ACTION, NULL, "Diamond 64 KB cartridge",         NULL, 10 },
 		{ "CRTB", ITEM_ACTION, NULL, "SpartaDOS X 64 KB cartridge",     NULL, 11 },
@@ -225,6 +329,25 @@ int SelectCartType(UBYTE* screen, int k)
 		{ "CRTJ", ITEM_ACTION, NULL, "Standard 8 KB 5200 cartridge",    NULL, 19 },
 		{ "CRTK", ITEM_ACTION, NULL, "Standard 4 KB 5200 cartridge",    NULL, 20 },
 		{ "CRTL", ITEM_ACTION, NULL, "Right slot 8 KB cartridge",       NULL, 21 },
+		{ "CRTM", ITEM_ACTION, NULL, "32 KB Williams cartridge",     	NULL, 22 },
+		{ "CRTN", ITEM_ACTION, NULL, "XEGS 256 KB cartridge",           NULL, 23 },
+		{ "CRTO", ITEM_ACTION, NULL, "XEGS 512 KB cartridge",           NULL, 24 },
+		{ "CRTP", ITEM_ACTION, NULL, "XEGS 1 MB cartridge",             NULL, 25 },
+		{ "CRTQ", ITEM_ACTION, NULL, "MegaCart 16 KB cartridge",        NULL, 26 },
+		{ "CRTR", ITEM_ACTION, NULL, "MegaCart 32 KB cartridge",        NULL, 27 },
+		{ "CRTS", ITEM_ACTION, NULL, "MegaCart 64 KB cartridge",        NULL, 28 },
+		{ "CRTT", ITEM_ACTION, NULL, "MegaCart 128 KB cartridge",       NULL, 29 },
+		{ "CRTU", ITEM_ACTION, NULL, "MegaCart 256 KB cartridge",       NULL, 30 },
+		{ "CRTV", ITEM_ACTION, NULL, "MegaCart 512 KB cartridge",       NULL, 31 },
+		{ "CRTW", ITEM_ACTION, NULL, "MegaCart 1 MB cartridge",         NULL, 32 },
+		{ "CRTX", ITEM_ACTION, NULL, "Switchable XEGS 32 KB cartridge", NULL, 33 },
+		{ "CRTY", ITEM_ACTION, NULL, "Switchable XEGS 64 KB cartridge", NULL, 34 },
+		{ "CRTZ", ITEM_ACTION, NULL, "Switchable XEGS 128 KB cartridge",NULL, 35 },
+		{ "CRU0", ITEM_ACTION, NULL, "Switchable XEGS 256 KB cartridge",NULL, 36 },
+		{ "CRU1", ITEM_ACTION, NULL, "Switchable XEGS 512 KB cartridge",NULL, 37 },
+		{ "CRU2", ITEM_ACTION, NULL, "Switchable XEGS 1 MB cartridge",  NULL, 38 },
+		{ "CRU3", ITEM_ACTION, NULL, "Phoenix 8 KB cartridge",          NULL, 39 },
+		{ "CRU4", ITEM_ACTION, NULL, "Blizzard 16 KB cartridge",        NULL, 40 },
 		MENU_END
 	};
 
@@ -247,7 +370,6 @@ int SelectCartType(UBYTE* screen, int k)
 	return CART_NONE;
 }
 
-
 void CartManagement()
 {
 	static tMenuItem menu_array[] =
@@ -451,15 +573,34 @@ int RunExe()
 	return ret;
 }
 
+int LoadTape()
+{
+	char tapename[FILENAME_MAX+1];
+	int ret = FALSE;
+
+	if (!curr_tape_dir[0])
+	  strcpy(curr_tape_dir, atari_exe_dir);
+	if (ui_driver->fGetLoadFilename(curr_tape_dir, tapename)) {
+		ret = CASSETTE_Insert(tapename);
+		if (! ret) {
+			/* display log to a window */
+		}
+	}
+
+	return ret;
+}
+
 void AtariSettings()
 {
 	static tMenuItem menu_array[] =
 	{
 		{ "NBAS", ITEM_ENABLED|ITEM_CHECK, NULL, "Disable BASIC when booting Atari:", NULL, 0 },
-		{ "RTM8", ITEM_ENABLED|ITEM_CHECK, NULL, "Enable R-Time 8:",                  NULL, 1 },
-		{ "SIOP", ITEM_ENABLED|ITEM_CHECK, NULL, "SIO patch (fast disk access):",     NULL, 2 },
-		{ "HDEV", ITEM_ENABLED|ITEM_CHECK, NULL, "H: device (hard disk):",            NULL, 3 },
-		{ "PDEV", ITEM_ENABLED|ITEM_CHECK, NULL, "P: device (printer):",              NULL, 4 },
+		{ "STRT", ITEM_ENABLED|ITEM_CHECK, NULL, "Boot from tape (hold Start):",      NULL, 1 },
+		{ "RTM8", ITEM_ENABLED|ITEM_CHECK, NULL, "Enable R-Time 8:",                  NULL, 2 },
+		{ "SIOP", ITEM_ENABLED|ITEM_CHECK, NULL, "SIO patch (fast disk access):",     NULL, 3 },
+		{ "HDEV", ITEM_ENABLED|ITEM_CHECK, NULL, "H: device (hard disk):",            NULL, 4 },
+		{ "PDEV", ITEM_ENABLED|ITEM_CHECK, NULL, "P: device (printer):",              NULL, 5 },
+		{ "RDEV", ITEM_ENABLED|ITEM_CHECK, NULL, "R: device (Atari850 via net):",              NULL, 6 },
 		MENU_END
 	};
 
@@ -470,22 +611,30 @@ void AtariSettings()
 			menu_array[0].flags |= ITEM_CHECKED;
 		else
 			menu_array[0].flags &= ~ITEM_CHECKED;
-		if(rtime_enabled)
+		if(hold_start_on_reboot)
 			menu_array[1].flags |= ITEM_CHECKED;
 		else
 			menu_array[1].flags &= ~ITEM_CHECKED;
-		if(enable_sio_patch)
+		if(rtime_enabled)
 			menu_array[2].flags |= ITEM_CHECKED;
 		else
 			menu_array[2].flags &= ~ITEM_CHECKED;
-		if(enable_h_patch)
+		if(enable_sio_patch)
 			menu_array[3].flags |= ITEM_CHECKED;
 		else
 			menu_array[3].flags &= ~ITEM_CHECKED;
-		if(enable_p_patch)
+		if(enable_h_patch)
 			menu_array[4].flags |= ITEM_CHECKED;
 		else
 			menu_array[4].flags &= ~ITEM_CHECKED;
+		if(enable_p_patch)
+			menu_array[5].flags |= ITEM_CHECKED;
+		else
+			menu_array[5].flags &= ~ITEM_CHECKED;
+		if(enable_r_patch)
+			menu_array[6].flags |= ITEM_CHECKED;
+		else
+			menu_array[6].flags &= ~ITEM_CHECKED;
 
 		option = ui_driver->fSelect(NULL, TRUE, option, menu_array, NULL);
 
@@ -494,17 +643,24 @@ void AtariSettings()
 			disable_basic = !disable_basic;
 			break;
 		case 1:
-			rtime_enabled = !rtime_enabled;
+			hold_start_on_reboot = !hold_start_on_reboot;
+			hold_start = hold_start_on_reboot;
 			break;
 		case 2:
-			enable_sio_patch = !enable_sio_patch;
+			rtime_enabled = !rtime_enabled;
 			break;
 		case 3:
-			enable_h_patch = !enable_h_patch;
+			enable_sio_patch = !enable_sio_patch;
 			break;
 		case 4:
+			enable_h_patch = !enable_h_patch;
+			break;
+		case 5:
 			enable_p_patch = !enable_p_patch;
 			break;
+		case 6:
+			enable_r_patch = !enable_r_patch;
+			break;
 		}
 	} while (option >= 0);
 	Atari800_UpdatePatches();
@@ -569,19 +725,85 @@ void SelectArtifacting()
 	}
 }
 
-void SetSoundType()
+int SoundSettings()
 {
-	char *msg;
-#ifdef STEREO
-	stereo_enabled = !stereo_enabled;
-	if (stereo_enabled)
-		msg = "Stereo sound output";
-	else
-		msg = " Mono sound output ";
-#else
-	msg = "Stereo sound was not compiled in";
+	int reboot_required = FALSE;
+
+	static tMenuItem menu_array[] =
+	{
+		{ "HFPO", ITEM_ENABLED|ITEM_CHECK, NULL, "High Fidelity POKEY:", NULL, 0 },
+		{ "STER", ITEM_ENABLED|ITEM_CHECK, NULL, "Dual POKEY (Stereo):", NULL, 1 },
+		{ "CONS", ITEM_ENABLED|ITEM_CHECK, NULL, "Speaker (Key Click):", NULL, 2 },
+		{ "SERI", ITEM_ENABLED|ITEM_CHECK, NULL, "Serial IO Sound    :", NULL, 3 },
+		MENU_END
+	};
+
+	int option = 0;
+
+	do {
+		if(enable_new_pokey)
+			menu_array[0].flags |= ITEM_CHECKED;
+		else
+			menu_array[0].flags &= ~ITEM_CHECKED;
+
+#ifdef STEREO_SOUND
+		if(stereo_enabled)
+			menu_array[1].flags |= ITEM_CHECKED;
+		else
 #endif
-	ui_driver->fMessage(msg);
+			menu_array[1].flags &= ~ITEM_CHECKED;
+
+#ifdef CONSOLE_SOUND
+		if(console_sound_enabled)
+			menu_array[2].flags |= ITEM_CHECKED;
+		else
+#endif
+			menu_array[2].flags &= ~ITEM_CHECKED;
+
+#ifdef SERIO_SOUND
+		if(serio_sound_enabled)
+			menu_array[3].flags |= ITEM_CHECKED;
+		else
+#endif
+			menu_array[3].flags &= ~ITEM_CHECKED;
+
+		option = ui_driver->fSelect(NULL, TRUE, option, menu_array, NULL);
+
+		switch (option) {
+		case 0:
+			enable_new_pokey = !enable_new_pokey;
+            Pokey_DoInit();
+            /* According the PokeySnd doc the POKEY switch can occur on
+               a cold-restart only */
+			reboot_required = TRUE;
+			ui_driver->fMessage("Will reboot to apply the change");
+			option = -1;	/* immediate exit from the while loop */
+			break;
+		case 1:
+#ifdef STEREO_SOUND
+			stereo_enabled = !stereo_enabled;
+#else
+			ui_driver->fMessage("Stereo sound support not compiled in");
+#endif
+			break;
+		case 2:
+#ifdef CONSOLE_SOUND
+			console_sound_enabled = !console_sound_enabled;
+#else
+			ui_driver->fMessage("Speaker sound support not compiled in");
+#endif
+			break;
+		case 3:
+#ifdef SERIO_SOUND
+			serio_sound_enabled = !serio_sound_enabled;
+#else
+			ui_driver->fMessage("SERIO sound support not compiled in");
+#endif
+			break;
+		}
+	} while (option >= 0);
+
+	return reboot_required;
 }
 
 void Screenshot(int interlaced)
@@ -600,15 +822,16 @@ void ui(UBYTE* screen)
 		{ "DISK", ITEM_ENABLED|ITEM_SUBMENU, NULL, "Disk Management",            "Alt+D",    MENU_DISK },
 		{ "CART", ITEM_ENABLED|ITEM_SUBMENU, NULL, "Cartridge Management",       "Alt+C",    MENU_CARTRIDGE },
 		{ "XBIN", ITEM_ENABLED|ITEM_FILESEL, NULL, "Run BIN file directly",      "Alt+R",    MENU_RUN },
+		{ "CASS", ITEM_ENABLED|ITEM_FILESEL, NULL, "Select tape image",          NULL,       MENU_CASSETTE },
 		{ "SYST", ITEM_ENABLED|ITEM_SUBMENU, NULL, "Select System",              "Alt+Y",    MENU_SYSTEM },
-		{ "STER", ITEM_ENABLED|ITEM_ACTION,  NULL, "Sound Mono/Stereo",          "Alt+O",    MENU_SOUND },
+		{ "SNDS", ITEM_ENABLED|ITEM_ACTION,  NULL, "Sound Settings", "Alt+O",    MENU_SOUND },
 		{ "SREC", ITEM_ENABLED|ITEM_ACTION,  NULL, "Sound Recording start/stop", "Alt+W",    MENU_SOUND_RECORDING },
 		{ "ARTF", ITEM_ENABLED|ITEM_SUBMENU, NULL, "Artifacting mode",           NULL,       MENU_ARTIF },
 		{ "SETT", ITEM_ENABLED|ITEM_SUBMENU, NULL, "Atari Settings",             NULL,       MENU_SETTINGS },
 		{ "SAVE", ITEM_ENABLED|ITEM_FILESEL, NULL, "Save State",                 "Alt+S",    MENU_SAVESTATE },
 		{ "LOAD", ITEM_ENABLED|ITEM_FILESEL, NULL, "Load State",                 "Alt+L",    MENU_LOADSTATE },
 		{ "PCXN", ITEM_ENABLED|ITEM_FILESEL, NULL, "PCX screenshot",             "F10",      MENU_PCX },
-		{ "PCXI", ITEM_ENABLED|ITEM_FILESEL, NULL, "PCX interlaced screenshot",  "Shift+F10",MENU_PCXI },
+/*		{ "PCXI", ITEM_ENABLED|ITEM_FILESEL, NULL, "PCX interlaced screenshot",  "Shift+F10",MENU_PCXI }, */
 		{ "CONT", ITEM_ENABLED|ITEM_ACTION,  NULL, "Back to emulated Atari",     "Esc",      MENU_BACK },
 		{ "REST", ITEM_ENABLED|ITEM_ACTION,  NULL, "Reset (Warm Start)",         "F5",       MENU_RESETW },
 		{ "REBT", ITEM_ENABLED|ITEM_ACTION,  NULL, "Reboot (Cold Start)",        "Shift+F5", MENU_RESETC },
@@ -662,6 +885,9 @@ void ui(UBYTE* screen)
 			if (RunExe())
 				done = TRUE;	/* reboot immediately */
 			break;
+		case MENU_CASSETTE:
+			LoadTape();
+			break;
 		case MENU_SYSTEM:
 			SelectSystem();
 			break;
@@ -672,7 +898,10 @@ void ui(UBYTE* screen)
 			AtariSettings();
 			break;
 		case MENU_SOUND:
-			SetSoundType();
+			if (SoundSettings()) {
+				Coldstart();
+				done = TRUE;	/* reboot immediately */
+			}
 			break;
 		case MENU_SOUND_RECORDING:
 			SoundRecording();
@@ -771,6 +1000,75 @@ int CrashMenu()
 
 /*
 $Log: ui.c,v $
+Revision 1.51  2003/12/16 18:26:30  pfusik
+new cartridge types: Phoenix and Blizzard
+
+Revision 1.50  2003/12/12 00:24:52  markgrebe
+Added enable for console and sio sound
+
+Revision 1.49  2003/11/22 23:26:19  joy
+cassette support improved
+
+Revision 1.48  2003/09/23 15:39:07  pfusik
+Rotate_Disks()
+
+Revision 1.47  2003/08/31 22:00:06  joy
+R: patch named as Atari850 emulation
+
+Revision 1.46  2003/05/28 19:54:58  joy
+R: device support (networking?)
+
+Revision 1.45  2003/03/03 10:16:01  joy
+multiple disk sets supported
+
+Revision 1.44  2003/02/27 17:42:08  pfusik
+new cartridge type
+
+Revision 1.43  2003/02/24 09:33:12  joy
+header cleanup
+
+Revision 1.42  2003/02/19 14:07:48  joy
+configure stuff cleanup
+
+Revision 1.41  2003/02/09 21:20:43  joy
+updated for global enable_new_pokey
+
+Revision 1.40  2003/02/09 13:17:29  joy
+switch Pokey cores on-the-fly
+
+Revision 1.39  2002/11/05 22:40:56  joy
+UI disk mounting fixes
+
+Revision 1.38  2002/09/16 11:22:06  pfusik
+five new cartridge types (Nir Dary)
+
+Revision 1.37  2002/09/05 08:35:11  pfusik
+seven new cartridge types (by Nir Dary)
+
+Revision 1.36  2002/08/15 16:57:20  pfusik
+1 MB XEGS cart
+
+Revision 1.34  2002/07/14 13:25:36  pfusik
+emulation of 576K and 1088K RAM machines
+
+Revision 1.33  2002/07/04 22:35:07  vasyl
+Added cassette support in main menu
+
+Revision 1.32  2002/07/04 12:41:38  pfusik
+emulation of 16K RAM machines: 400 and 600XL
+
+Revision 1.31  2002/06/23 21:42:09  joy
+SoundRecording() accessible from outside (atari_x11.c needs it)
+
+Revision 1.30  2002/03/30 06:19:28  vasyl
+Dirty rectangle scheme implementation part 2.
+All video memory accesses everywhere are going through the same macros
+in ANTIC.C. UI_BASIC does not require special handling anymore. Two new
+functions are exposed in ANTIC.H for writing to video memory.
+
+Revision 1.28  2002/01/10 16:46:42  joy
+new cartridge type added
+
 Revision 1.27  2001/11/18 19:35:59  fox
 fixed a bug: modification of string literals
 
@@ -839,3 +1137,4 @@ Revision 1.5  2001/03/18 06:34:58  knik
 WIN32 conditionals removed
 
 */
+
