@@ -1,4 +1,26 @@
-/* $Id: ui_basic.c,v 1.5 2001/11/29 12:36:42 joy Exp $ */
+/*
+ * ui_basic.c - main user interface
+ *
+ * Copyright (C) 1995-1998 David Firth
+ * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
 
 #include <stdio.h>
 #include <fcntl.h>
@@ -9,6 +31,7 @@
 #include <sys/stat.h>
 #include "rt-config.h"
 #include "atari.h"
+#include "antic.h"
 #include "list.h"
 #include "ui.h"
 #include "log.h"
@@ -27,7 +50,7 @@
 #endif
 */
 
-#ifdef CURSES
+#ifdef USE_CURSES
 extern UBYTE curses_screen[24][40];
 #endif
 
@@ -141,7 +164,7 @@ int GetKeyPress(UBYTE * screen)
 
 void Plot(UBYTE * screen, int fg, int bg, int ch, int x, int y)
 {
-#ifndef CURSES
+#ifndef USE_CURSES
 	int offset = ascii_to_screen[(ch & 0x07f)] * 8;
 	int i;
 	int j;
@@ -157,9 +180,9 @@ void Plot(UBYTE * screen, int fg, int bg, int ch, int x, int y)
 
 		for (j = 0; j < 8; j++) {
 #ifdef USE_COLOUR_TRANSLATION_TABLE
-			*ptr++ = colour_translation_table[data & 0x80 ? fg : bg];
+			video_putbyte(ptr++, colour_translation_table[data & 0x80 ? fg : bg]);
 #else
-			*ptr++ = data & 0x80 ? fg : bg;
+			video_putbyte(ptr++, data & 0x80 ? fg : bg);
 #endif
 			data <<= 1;
 		}
@@ -287,16 +310,16 @@ void Box(UBYTE * screen, int fg, int bg, int x1, int y1, int x2, int y2)
 
 void ClearScreen(UBYTE * screen)
 {
-#ifndef CURSES
+#ifndef USE_CURSES
 	UBYTE *ptr;
 #ifdef USE_COLOUR_TRANSLATION_TABLE
-	memset(screen, colour_translation_table[0x00], ATARI_HEIGHT * ATARI_WIDTH);
+	video_memset(screen, colour_translation_table[0x00], ATARI_HEIGHT * ATARI_WIDTH);
 	for (ptr = screen + ATARI_WIDTH * 24 + 32; ptr < screen + ATARI_WIDTH * (24 + 192); ptr += ATARI_WIDTH)
-		memset(ptr, colour_translation_table[0x94], 320);
+		video_memset(ptr, colour_translation_table[0x94], 320);
 #else
-	memset(screen, 0x00, ATARI_HEIGHT * ATARI_WIDTH);
+	video_memset(screen, 0x00, ATARI_HEIGHT * ATARI_WIDTH);
 	for (ptr = screen + ATARI_WIDTH * 24 + 32; ptr < screen + ATARI_WIDTH * (24 + 192); ptr += ATARI_WIDTH)
-		memset(ptr, 0x94, 320);
+		video_memset(ptr, 0x94, 320);
 #endif
 #else
 	int x;
@@ -706,8 +729,12 @@ int FileSelector(UBYTE * screen, char *directory, char *full_filename)
 				}
 				else if (item >= nitems) {	/* Scroll Left */
 					if ((offset - NROWS) >= 0)
+					{
 						offset -= NROWS;
-					item = item % nitems;
+						item = item % nitems;
+					}
+					else
+						item = 0;
 				}
 				else if (item == -2) {	/* Next directory */
 					DIR *dr;
@@ -813,7 +840,7 @@ void AboutEmulator(UBYTE * screen)
 	CenterPrint(screen, 0x9a, 0x94, ATARI_TITLE, 1);
 	CenterPrint(screen, 0x9a, 0x94, "Copyright (c) 1995-1998 David Firth", 2);
 	CenterPrint(screen, 0x9a, 0x94, "and", 3);
-	CenterPrint(screen, 0x9a, 0x94, "(c)1998-2001 Atari800 Development Team", 4);
+	CenterPrint(screen, 0x9a, 0x94, "(c)1998-2003 Atari800 Development Team", 4);
 	CenterPrint(screen, 0x9a, 0x94, "See CREDITS file for details.", 5);
 	CenterPrint(screen, 0x9a, 0x94, "http://atari800.atari.org/", 7);
 
@@ -990,6 +1017,24 @@ void BasicUIInit()
 
 /*
 $Log: ui_basic.c,v $
+Revision 1.10  2003/02/24 09:33:13  joy
+header cleanup
+
+Revision 1.9  2003/02/08 23:52:17  joy
+little cleanup
+
+Revision 1.8  2003/01/27 13:14:51  joy
+Jason's changes: either PAGED_ATTRIB support (mostly), or just clean up.
+
+Revision 1.7  2002/06/12 06:40:41  vasyl
+Fixed odd behavior of Up button on the first item in file selector
+
+Revision 1.6  2002/03/30 06:19:28  vasyl
+Dirty rectangle scheme implementation part 2.
+All video memory accesses everywhere are going through the same macros
+in ANTIC.C. UI_BASIC does not require special handling anymore. Two new
+functions are exposed in ANTIC.H for writing to video memory.
+
 Revision 1.5  2001/11/29 12:36:42  joy
 copyright notice updated
 
