@@ -1,6 +1,5 @@
-/* (C) 2001  Vasyl Tsvirkunov */
-#ifndef A800_KEYBOARD_H
-#define A800_KEYBOARD_H
+#ifndef _KEYBOARD_H_
+#define _KEYBOARD_H_
 
 int prockb(void);
 int initinput(void);
@@ -12,6 +11,7 @@ void releasebutton(short);
 
 void tapscreen(short x, short y);
 void untapscreen(short x, short y);
+void dragscreen(short x, short y);
 
 extern int console;
 extern int trig0;
@@ -27,4 +27,7 @@ extern int currentKeyboardColor;
 /* Packed bitmap 240x80 - five rows of buttons */
 extern unsigned long* kbd_image;
 
-#endif
+/* Use virtual joystick */
+extern int virtual_joystick;
+
+#endif /* _KEYBOARD_H_ */
